@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008,
-   2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 1999-2005, 2007-2012, 2014-2018 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <frm.h>
 
@@ -31,9 +29,6 @@ static int align = 0;      /* Tidy mode. -t option. */
 #define IS_NEW  0x100
 static int select_attribute;
 
-const char *program_version = "frm (" PACKAGE_STRING ")";
-static char doc[] = N_("GNU frm -- display From: lines.");
-
 static struct attr_tab {
   char *name;      /* Attribute name */
   int code;        /* Corresponding IS_.* flag */
@@ -87,7 +82,7 @@ prepare_attrs (void)
 /* Translates the textual status representation to the corresponding
    IS_.* flag */
 static int
-decode_attr (char *arg)
+decode_attr (char const *arg)
 {
   struct attr_tab *p;
   int len = strlen (arg);
@@ -114,105 +109,81 @@ decode_attr (char *arg)
 }
 
 
-
-static struct argp_option options[] = {
-  {"debug",  'd', NULL,   0, N_("enable debugging output"), 0},
-  {"field",  'f', N_("NAME"), 0, N_("header field to display"), 0},
-  {"to",     'l', NULL,   0, N_("include the To: information"), 0},
-  {"number", 'n', NULL,   0, N_("display message numbers"), 0},
-  {"Quiet",  'Q', NULL,   0, N_("do not display headers"), 0},
-  {"query",  'q', NULL,   0, N_("print a message if the mailbox contains some unread mail"), 0},
-  {"summary",'S', NULL,   0, N_("print a summary of messages"), 0},
-  {"status", 's', N_("STATUS"), 0,
-   /* TRANSLATORS: Please do *not* translate the words "new", "unread",
-      "old" and "read". They are keywords. */
-   N_("select messages with the specific attribute:"
-      " new, unread, old (same as unread) or read (or any unambiguous"
-      " abbreviation of these)"),
-   0},
-  {"align",  't', NULL,   0, N_("tidy mode: align subject lines"), 0},
-  {0, 0, 0, 0}
-};
-
-static error_t
-parse_opt (int key, char *arg, struct argp_state *state)
+static void
+cli_show_field (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 {
-  switch (key)
-    {
-    case 'd':
-      frm_debug++;
-      break;
-
-    case 'f':
-      show_field = arg;
-      align = 0;
-      break;
-
-    case 'l':
-      show_to = 1;
-      break;
-
-    case 'n':
-      show_number = 1;
-      break;
-
-    case 'Q':
-      /* Very silent.  */
-      be_quiet += 2;
-      break;
-
-    case 'q':
-      be_quiet++;
-      show_query = 1;
-      break;
-
-    case 'S':
-      show_summary = 1;
-      break;
-
-    case 's':
-      select_attribute = decode_attr (arg);
-      break;
-      
-    case 't':
-      align = 1;
-      break;
-
-    case ARGP_KEY_FINI:
-      {
-	size_t s;
-	if (align && (s = util_getcols ()))
-	  init_output (s);
-	else
-	  init_output (0);
-      }
-      break;
-      
-    default: 
-      return ARGP_ERR_UNKNOWN;
-    }
-  return 0;
+  show_field = mu_strdup (arg);
+  align = 0;
 }
 
-static struct argp argp = {
+static void
+cli_Quiet (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
+{
+  be_quiet += 2;
+}
+
+static void
+cli_query (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
+{
+  be_quiet++;
+  show_query = 1;
+}
+
+static void
+cli_status (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
+{
+  select_attribute = decode_attr (arg);
+}
+
+static struct mu_option frm_options[] = {
+  { "debug",  'd', NULL,   MU_OPTION_DEFAULT,
+    N_("enable debugging output"),
+    mu_c_incr, &frm_debug },
+  { "field",  'f', N_("NAME"), MU_OPTION_DEFAULT,
+    N_("header field to display"),
+    mu_c_string, &show_field, cli_show_field },
+  { "to",     'l', NULL,   MU_OPTION_DEFAULT,
+    N_("include the To: information"),
+    mu_c_bool, &show_to },
+  { "number", 'n', NULL,   MU_OPTION_DEFAULT,
+    N_("display message numbers"),
+    mu_c_bool, &show_number },
+  { "Quiet",  'Q', NULL,   MU_OPTION_DEFAULT,
+    N_("do not display headers"),
+    mu_c_int, &be_quiet, cli_Quiet },
+  { "query",  'q', NULL,   MU_OPTION_DEFAULT,
+    N_("print a message if the mailbox contains some unread mail"),
+    mu_c_int, &be_quiet, cli_query },
+  { "summary",'S', NULL,   MU_OPTION_DEFAULT,
+    N_("print a summary of messages"), 
+    mu_c_bool, &show_summary },
+  { "status", 's', N_("STATUS"), 0,
+    /* TRANSLATORS: Please do *not* translate the words "new", "unread",
+       "old" and "read". They are keywords. */
+    N_("select messages with the specific attribute:"
+       " new, unread, old (same as unread) or read (or any unambiguous"
+       " abbreviation of these)"),
+    mu_c_string, NULL, cli_status},
+  { "align",  't', NULL,   MU_OPTION_DEFAULT,
+    N_("tidy mode: align subject lines"),
+    mu_c_bool, &align },
+  MU_OPTION_END
+}, *options[] = { frm_options, NULL };
+
+static struct mu_cli_setup cli = {
   options,
-  parse_opt,
-  N_("[URL ...]"),
-  doc,
   NULL,
-  NULL, NULL
+  N_("GNU frm -- display From: lines."),
+  N_("[URL ...]"),
 };
 
-static const char *frm_argp_capa[] = {
-  "common",
+static char *capa[] = {
   "debug",
-  "license",
   "mailbox",
   "locking",
   NULL
 };
-
-
+
 static struct
 {
   size_t new;
@@ -265,21 +236,22 @@ frm (char *mailbox_name)
   int status;
 
   selected = 0;
+  counter.new = counter.read = counter.unread = 0;
   frm_scan (mailbox_name, frm_select, &total);
   
   if (show_summary)
     {
       if (total == 0)
-	printf (_("Folder contains no messages."));
+	mu_printf (_("Folder contains no messages."));
       else
 	{
 	  char *delim = "";
       	  
-	  printf (_("Folder contains "));
+	  mu_printf (_("Folder contains "));
 	
 	  if (counter.new)
 	    {
-	      printf (ngettext ("%lu new message",
+	      mu_printf (ngettext ("%lu new message",
  				"%lu new messages",
 				counter.new),
 		      (u_long) counter.new);
@@ -288,9 +260,9 @@ frm (char *mailbox_name)
 	  
 	  if (counter.unread)
 	    {
-	      printf ("%s", delim);
+	      mu_printf ("%s", delim);
 	      
-	      printf (ngettext ("%lu unread message",
+	      mu_printf (ngettext ("%lu unread message",
 				"%lu unread messages",
 				counter.unread),
 		      (u_long) counter.unread);
@@ -299,9 +271,9 @@ frm (char *mailbox_name)
 	  
 	  if (counter.read)
 	    {
-	      printf ("%s", delim);
+	      mu_printf ("%s", delim);
 	      
-	      printf (ngettext ("%lu read message",
+	      mu_printf (ngettext ("%lu read message",
 				"%lu read messages",
 				counter.read),
 		      (u_long) counter.read);
@@ -313,16 +285,16 @@ frm (char *mailbox_name)
 	      Leave it as it is unless your language requires to reorder
 	      the parts of speach in the message
 	  */
-	  printf (_("."));
+	  mu_printf (_("."));
 	}
-      printf ("\n");
+      mu_printf ("\n");
     }
   else if (show_query)
     {
       if (total > 0)
-	printf (_("There are messages in that folder.\n"));
+	mu_printf (_("There are messages in that folder.\n"));
       else
-	printf (_("No messages in that folder!\n"));
+	mu_printf (_("No messages in that folder!\n"));
     }
   
   /* EXIT STATUS
@@ -344,8 +316,8 @@ frm (char *mailbox_name)
 int
 main (int argc, char **argv)
 {
-  int c;
   int status = 0;
+  size_t s;
 
   /* Native Language Support */
   MU_APP_INIT_NLS ();
@@ -354,25 +326,30 @@ main (int argc, char **argv)
   
   /* register the formats.  */
   mu_register_all_mbox_formats ();
-#ifdef WITH_TLS
-  mu_gocs_register ("tls", mu_tls_module_init);
-#endif
-  
-  mu_argp_init (program_version, NULL);
-  if (mu_app_init (&argp, frm_argp_capa, NULL, argc, argv, 0, &c, NULL))
-    exit (1);
 
-  /* have an argument */
-  if (c == argc)
-    status = frm (NULL);
-  else if (c + 1 == argc)
-    status = frm (argv[c]);
+  mu_auth_register_module (&mu_auth_tls_module);
+  mu_cli (argc, argv, &cli, capa, NULL, &argc, &argv);
+
+  if (align && (s = util_getcols ()))
+    init_output (s);
   else
-    for (; c < argc; c++)
-      {
-	printf ("%s:\n", argv[c]);
-	status = frm (argv[c]);
-      }
-
+    init_output (0);
+  
+  /* have an argument */
+  if (argc == 0)
+    status = frm (NULL);
+  else if (argc == 1)
+    status = frm (argv[0]);
+  else
+    {
+      int i;
+      
+      for (i = 0; i < argc; i++)
+	{
+	  mu_printf ("%s:\n", argv[i]);
+	  status = frm (argv[i]);
+	}
+    }
+  
   return status;
 }
