@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail -*- c -*-
-   Copyright (C) 1999-2000, 2005, 2007-2008, 2010-2012, 2014-2017 Free
+   Copyright (C) 1999-2000, 2005, 2007-2008, 2010-2012, 2014-2018 Free
    Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -52,10 +52,6 @@ extern int mu_debug_line_info;
   ((a) == 0 ? MU_DEBUG_LEVEL_UPTO (b) :					\
               MU_DEBUG_LEVEL_UPTO (b) & ~MU_DEBUG_LEVEL_UPTO ((a) - 1))
 
-
-struct sockaddr;
-char *mu_sockaddr_to_astr (const struct sockaddr *sa, int salen);
-
 
 size_t mu_debug_register_category (char *name);
 size_t mu_debug_next_handle (void);
