@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2009, 2010, 2011 Free
+   Copyright (C) 1999-2001, 2005, 2007, 2009-2012, 2014-2018 Free
    Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -61,17 +61,16 @@ extern int  mu_folder_enumerate      (mu_folder_t, const char *,
 extern int  mu_folder_lsub           (mu_folder_t, const char *, const char *,
 				      mu_list_t *);
 
-/* Stream settings.  */
-extern int  mu_folder_get_stream     (mu_folder_t, mu_stream_t *)
-                                       __attribute__ ((deprecated));
-extern int  mu_folder_get_streamref  (mu_folder_t, mu_stream_t *);
-  
-extern int  mu_folder_set_stream     (mu_folder_t, mu_stream_t);
-
   /* Match function */
 extern int mu_folder_set_match (mu_folder_t folder, mu_folder_match_fp pmatch);
 extern int mu_folder_get_match (mu_folder_t folder,
 				mu_folder_match_fp *pmatch);
+
+  /* Two often used matchers: */
+  /* 1. The default: IMAP-style wildcards: */
+extern int mu_folder_imap_match (const char *name, void *pattern, int flags);
+  /* 2. UNIX-style glob(7) wildcards: */
+extern int mu_folder_glob_match (const char *name, void *pattern, int flags);
   
   /* Notifications.  */
 extern int  mu_folder_get_observable (mu_folder_t, mu_observable_t *);
@@ -92,6 +91,8 @@ extern int  mu_folder_get_property   (mu_folder_t, mu_property_t *);
 extern int  mu_folder_decrement      (mu_folder_t);
 
 extern void mu_list_response_free    (void *data);
+
+int _mu_fsfolder_init (mu_folder_t folder);
   
 #ifdef __cplusplus
 }
