@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2008, 2009, 2010, 2011
-   Free Software Foundation, Inc.
+   Copyright (C) 1999-2001, 2005, 2007-2012, 2014-2018 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -61,6 +61,7 @@ extern "C" {
 					     "file:///..." */
 #define MU_URL_PARSE_DSLASH_OPTIONAL 0x0040 /* Double-slash after scheme:
 					       part is optional */
+#define MU_URL_PARSE_LOCAL        0x0080  /* Local URL: no host part */   
   
 #define MU_URL_PARSE_DEFAULT \
   (MU_URL_PARSE_HEXCODE|MU_URL_PARSE_HIDEPASS|MU_URL_PARSE_PORTSRV|\
