@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2004, 2005, 2006, 2007, 2008,
-   2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 1999-2002, 2004-2012, 2014-2018 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -47,3 +47,7 @@ mu_record_t mu_mh_record = NULL;
 mu_record_t mu_maildir_record = NULL;
 #endif
 
+#ifndef WITH_TLS
+mu_record_t mu_smtps_record = NULL;
+#endif
+
