@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2003, 2005-2007, 2009-2012 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2003, 2005-2007, 2009-2012, 2014-2018 Free
+   Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,6 @@
 # include <config.h>
 #endif
 
-#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -31,6 +30,7 @@
 #include <mailutils/mu_auth.h>
 #include <mailutils/util.h>
 #include <mailutils/parse822.h>
+#include <mailutils/errno.h>
 
 /*
  * Functions used to convert unix mailbox/user names into RFC822 addr-specs.
@@ -46,6 +46,13 @@ mu_set_user_email (const char *candidate)
   size_t emailno = 0;
   char *email = NULL;
   const char *domain = NULL;
+
+  if (!candidate)
+    {
+      free (mu_user_email);
+      mu_user_email = NULL;
+      return 0;
+    }
   
   if ((err = mu_address_create (&addr, candidate)) != 0)
     return err;
@@ -61,14 +68,24 @@ mu_set_user_email (const char *candidate)
 
   if ((err = mu_address_aget_email (addr, 1, &email)) != 0)
     goto cleanup;
-
-  if (mu_user_email)
-    free (mu_user_email);
+  else if (email == NULL)
+    {
+      err = MU_ERR_NOENT;
+      goto cleanup;
+    }
+  
+  free (mu_user_email);
 
   mu_user_email = email;
 
-  if ((err = mu_address_sget_domain (addr, 1, &domain)) == 0)
-    mu_set_user_email_domain (domain);
+  if ((err = mu_address_sget_domain (addr, 1, &domain)) != 0)
+    goto cleanup;
+  else if (domain == NULL)
+    {
+      err = MU_ERR_NOENT;
+      goto cleanup;
+    }
+  mu_set_user_email_domain (domain);
   
 cleanup:
   mu_address_destroy (&addr);
@@ -81,16 +98,18 @@ static char *mu_user_email_domain = 0;
 int
 mu_set_user_email_domain (const char *domain)
 {
-  char *d = NULL;
+  char *d;
   
-  if (!domain)
-    return EINVAL;
+  if (domain)
+    {
+      d = strdup (domain);
+
+      if (!d)
+	return ENOMEM;
+    }
+  else
+    d = NULL;
   
-  d = strdup (domain);
-
-  if (!d)
-    return ENOMEM;
-
   if (mu_user_email_domain)
     free (mu_user_email_domain);
 
@@ -154,7 +173,7 @@ mu_get_user_email (const char *name)
 
   if (!name)
     {
-      struct mu_auth_data *auth = mu_get_auth_by_uid (getuid ());
+      struct mu_auth_data *auth = mu_get_auth_by_uid (geteuid ());
       if (!auth)
 	{
 	  errno = EINVAL;
@@ -163,6 +182,11 @@ mu_get_user_email (const char *name)
       name = tmpname = strdup(auth->name);
       if (auth)
 	mu_auth_data_free (auth);
+      if (!name)
+        {
+          errno = ENOMEM;
+          return NULL;
+        }
     }
 
   status = mu_get_user_email_domain (&domainpart);
