@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 2008-2012, 2014-2018 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 #endif
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -43,6 +44,7 @@
 #include <mailutils/acl.h>
 #include <mailutils/sockaddr.h>
 #include <mailutils/url.h>
+#include <mailutils/util.h>
 
 typedef RETSIGTYPE (*mu_sig_handler_t) (int);
 
@@ -75,7 +77,7 @@ struct _mu_m_server
   mu_list_t srvlist;             /* A list of configured mu_ip_server_t
 				    objects. It is cleared after the objects
 				    are opened and attached to the server. */
-  
+  mu_m_server_preflight_fp preflight; /* Pre-flight check function */
   mu_m_server_handler_fp conn;   /* Connection handler function. */
   mu_m_server_handler_fp prefork;/* Pre-fork function. */
   void *data;                    /* User-supplied data for conn and prefork. */
@@ -91,7 +93,7 @@ struct _mu_m_server
   struct mu_sockaddr_hints hints; /* Default address hints. */
   time_t timeout;                /* Default idle timeout. */
   mu_acl_t acl;                  /* Global access control list. */
-
+  
   sigset_t sigmask;              /* A set of signals to handle by the
 				    m-server.  */
   mu_sig_handler_t sigtab[NSIG]; /* Keeps old signal handlers. */
@@ -214,7 +216,7 @@ m_server_cleanup (void *item, void *data)
 	mu_diag_output (MU_DIAG_ERR,
 			"process %lu terminated (cause unknown)",
 			(unsigned long) datp->pid);
-      return 1;
+      return MU_ERR_USER0;
     }
   return 0;
 }
@@ -229,7 +231,7 @@ mu_m_server_idle (void *server_data MU_ARG_UNUSED)
       need_cleanup = 0;
       while ( (ex.pid = waitpid (-1, &ex.status, WNOHANG)) > 0)
 	/* Iterate over all m-servers and notify them about the fact. */
-	mu_list_do (m_server_list, m_server_cleanup, &ex);
+	mu_list_foreach (m_server_list, m_server_cleanup, &ex);
     }
   return stop;
 }
@@ -343,9 +345,12 @@ mu_m_server_set_max_children (mu_m_server_t srv, size_t num)
 int
 mu_m_server_set_pidfile (mu_m_server_t srv, const char *pidfile)
 {
+  char *p = strdup (pidfile);
+  if (!p)
+    return errno;
   free (srv->pidfile);
-  srv->pidfile = strdup (pidfile);
-  return srv->pidfile ? 0 : errno;
+  srv->pidfile = p;
+  return 0;
 }
 
 int
@@ -361,6 +366,12 @@ mu_m_server_set_strexit (mu_m_server_t srv, const char *(*fun) (int))
   srv->strexit = fun;
 }
 
+void
+mu_m_server_set_preflight (mu_m_server_t srv, mu_m_server_preflight_fp fun)
+{
+  srv->preflight = fun;
+}
+
 int
 mu_m_server_get_srvlist (mu_m_server_t srv, mu_list_t *plist)
 {
@@ -388,7 +399,7 @@ mu_m_server_get_hints (mu_m_server_t srv, struct mu_sockaddr_hints *hints)
 {
   if (!hints)
     return EINVAL;
-  memcpy (hints, &srv->hints, sizeof (hints));
+  memcpy (hints, &srv->hints, sizeof (*hints));
   return 0;
 }
 
@@ -450,8 +461,8 @@ static int m_srv_conn (int fd, struct sockaddr *sa, int salen,
 		       void *server_data, void *call_data,
 		       mu_ip_server_t srv);
 
-static struct mu_srv_config *
-add_server (mu_m_server_t msrv, struct mu_sockaddr *s, int type)
+struct mu_srv_config *
+mu_m_server_listen (mu_m_server_t msrv, struct mu_sockaddr *s, int type)
 {
   mu_ip_server_t tcpsrv;
   struct mu_srv_config *pconf;
@@ -476,9 +487,9 @@ add_server (mu_m_server_t msrv, struct mu_sockaddr *s, int type)
 }
 
 void
-mu_m_server_configured_count (mu_m_server_t msrv, size_t count)
+mu_m_server_configured_count (mu_m_server_t msrv, size_t *count)
 {
-  mu_list_count (msrv->srvlist, &count);
+  mu_list_count (msrv->srvlist, count);
 }
 
 void
@@ -502,7 +513,7 @@ mu_m_server_begin (mu_m_server_t msrv)
 	  {
 	    struct mu_sockaddr *next = ta->next;
 	    ta->next = ta->prev = NULL;
-	    add_server (msrv, ta, msrv->deftype);
+	    mu_m_server_listen (msrv, ta, msrv->deftype);
 	    ta = next;
 	  }
     }
@@ -516,6 +527,7 @@ mu_m_server_begin (mu_m_server_t msrv)
 	  mu_error (_("failed to become a daemon: %s"), mu_strerror (errno));
 	  exit (EXIT_FAILURE);
 	}
+      mu_onexit_reset ();
     }
 
   if (msrv->pidfile)
@@ -559,6 +571,7 @@ mu_m_server_destroy (mu_m_server_t *pmsrv)
 {
   mu_m_server_t msrv = *pmsrv;
   mu_list_remove (m_server_list, msrv);
+  mu_list_destroy (&msrv->srvlist);  
   mu_server_destroy (&msrv->server);
   free (msrv->child_pid);
   /* FIXME: Send processes the TERM signal here?*/
@@ -588,17 +601,16 @@ tcp_conn_free (void *conn_data, void *server_data)
 }
 
 static int
-_open_conn (void *item, void *data)
+open_connection (mu_ip_server_t tcpsrv, mu_m_server_t msrv)
 {
-  mu_ip_server_t tcpsrv = item;
-  mu_m_server_t msrv = data;
   int rc = mu_ip_server_open (tcpsrv);
   if (rc)
     {
       mu_error (_("cannot open connection on %s: %s"),
 		mu_ip_server_addrstr (tcpsrv), mu_strerror (rc));
-      return 0;
+      return rc;
     }
+
   rc = mu_server_add_connection (msrv->server,
 				 mu_ip_server_get_fd (tcpsrv),
 				 tcpsrv,
@@ -608,9 +620,8 @@ _open_conn (void *item, void *data)
       mu_error (_("cannot add connection %s: %s"),
 		mu_ip_server_addrstr (tcpsrv), mu_strerror (rc));
       mu_ip_server_shutdown (tcpsrv);
-      mu_ip_server_destroy (&tcpsrv);
     }
-  return 0;
+  return rc;
 }  
 
 int
@@ -618,16 +629,36 @@ mu_m_server_run (mu_m_server_t msrv)
 {
   int rc;
   size_t count;
-  mode_t saved_umask = umask (0117);
-  mu_list_do (msrv->srvlist, _open_conn, msrv);
+  mode_t saved_umask;
+  mu_iterator_t itr;
+  
+  saved_umask = umask (0117);
+  mu_list_get_iterator (msrv->srvlist, &itr);
+  for (mu_iterator_first (itr); !mu_iterator_is_done (itr); mu_iterator_next (itr))
+    {
+      mu_ip_server_t tcpsrv;
+      mu_iterator_current (itr, (void**) &tcpsrv);
+      if (open_connection (tcpsrv, msrv))
+	{
+	  mu_iterator_ctl (itr, mu_itrctl_delete_nd, NULL);
+	  mu_ip_server_destroy (&tcpsrv);
+	}
+    }
   umask (saved_umask);
-  mu_list_destroy (&msrv->srvlist);
+  mu_iterator_destroy (&itr);
+  
   MU_ASSERT (mu_server_count (msrv->server, &count));
   if (count == 0)
     {
       mu_error (_("no servers configured: exiting"));
       exit (1);
     }
+  if (msrv->preflight && msrv->preflight (msrv))
+    {
+      mu_error (_("%s: preflight check failed"), msrv->ident);
+      return MU_ERR_FAILURE;
+    }
+  
   if (msrv->ident)
     mu_diag_output (MU_DIAG_INFO, _("%s started"), msrv->ident);
   rc = mu_server_run (msrv->server);
@@ -636,8 +667,6 @@ mu_m_server_run (mu_m_server_t msrv)
     mu_diag_output (MU_DIAG_INFO, _("%s terminated"), msrv->ident);
   return rc;
 }
-
-
 
 int
 mu_m_server_check_acl (mu_m_server_t msrv, struct sockaddr *s, int salen)
@@ -645,10 +674,12 @@ mu_m_server_check_acl (mu_m_server_t msrv, struct sockaddr *s, int salen)
   if (msrv->acl)
     {
       mu_acl_result_t res;
-      int rc = mu_acl_check_sockaddr (msrv->acl, s, salen, &res);
+      int rc;
+
+      rc = mu_acl_check_sockaddr (msrv->acl, s, salen, &res);
       if (rc)
 	{
-	  char *p = mu_sockaddr_to_astr (s, salen);
+	  char *p = mu_sys_sockaddr_to_astr (s, salen);
 	  mu_error (_("access from %s blocked: cannot check ACLs: %s"),
 		    p, mu_strerror (rc));
 	  free (p);
@@ -658,7 +689,7 @@ mu_m_server_check_acl (mu_m_server_t msrv, struct sockaddr *s, int salen)
 	{
 	case mu_acl_result_undefined:
 	  {
-	    char *p = mu_sockaddr_to_astr (s, salen);
+	    char *p = mu_sys_sockaddr_to_astr (s, salen);
 	    mu_diag_output (MU_DIAG_INFO,
 			    _("%s: undefined ACL result; access allowed"),
 			    p);
@@ -671,7 +702,7 @@ mu_m_server_check_acl (mu_m_server_t msrv, struct sockaddr *s, int salen)
 	      
 	case mu_acl_result_deny:
 	  {
-	    char *p = mu_sockaddr_to_astr (s, salen);
+	    char *p = mu_sys_sockaddr_to_astr (s, salen);
 	    mu_error (_("access from %s blocked"), p);
 	    free (p);
 	    return 1;
@@ -688,7 +719,7 @@ m_srv_conn (int fd, struct sockaddr *sa, int salen,
 {
   int status;
   struct mu_srv_config *pconf = server_data;
-  
+
   if (mu_m_server_check_acl (pconf->msrv, sa, salen))
     return 0;
 
@@ -783,7 +814,7 @@ server_block_begin (const char *arg, mu_m_server_t msrv, void **pdata)
 			"only the first is used"), arg);
       mu_sockaddr_free (s->next);
     }
-  *pdata = add_server (msrv, s, msrv->deftype);
+  *pdata = mu_m_server_listen (msrv, s, msrv->deftype);
   return 0;
 }
 
@@ -852,14 +883,14 @@ get_port (const char *p)
 	      return 1;
 	    }
 	  
-	  return htons (n);
+	  return n;
 	}
       else
 	{
 	  struct servent *sp = getservbyname (p, "tcp");
 	  if (!sp)
 	    return 0;
-	  return sp->s_port;
+	  return ntohs (sp->s_port);
 	}
     }
   return 0;
@@ -881,24 +912,26 @@ _cb_port (void *data, mu_config_value_t *val)
 }
 
 static struct mu_cfg_param dot_server_cfg_param[] = {
-  { "max-children", mu_cfg_size,
+  { "max-children", mu_c_size,
     NULL, mu_offsetof (struct _mu_m_server,max_children), NULL,
     N_("Maximum number of children processes to run simultaneously.") },
   { "mode", mu_cfg_callback,
     NULL, mu_offsetof (struct _mu_m_server,mode), _cb_daemon_mode,
-    N_("Set daemon mode (either inetd (or interactive) or daemon)."),
-    N_("mode") },
-  { "foreground", mu_cfg_bool,
+    N_("Set operation mode."),
+    /* TRANSLATORS: Words to the right of : are keywords. */
+    N_("mode: inetd|interactive|daemom") },
+  { "foreground", mu_c_bool,
     NULL, mu_offsetof (struct _mu_m_server, foreground), NULL,
     N_("Run in foreground.") },
-  { "pidfile", mu_cfg_string,
+  { "pidfile", mu_c_string,
     NULL, mu_offsetof (struct _mu_m_server,pidfile), NULL,
     N_("Store PID of the master process in this file."),
     N_("file") },
   { "port", mu_cfg_callback,
     NULL, mu_offsetof (struct _mu_m_server, hints), _cb_port,
-    N_("Default port number.") },
-  { "timeout", mu_cfg_time,
+    N_("Default port number."),
+    N_("arg: port number or service name") },
+  { "timeout", mu_c_time,
     NULL, mu_offsetof (struct _mu_m_server,timeout), NULL,
     N_("Set idle timeout.") },
   { "server", mu_cfg_section, NULL, 0, NULL,
@@ -909,13 +942,13 @@ static struct mu_cfg_param dot_server_cfg_param[] = {
 };
     
 static struct mu_cfg_param server_cfg_param[] = {
-  { "single-process", mu_cfg_bool, 
+  { "single-process", mu_c_bool, 
     NULL, mu_offsetof (struct mu_srv_config, single_process), NULL,
-    N_("Run this server in foreground.") },
-  { "transcript", mu_cfg_bool,
+    N_("Do not spawn sub-process to handle requests in this server.") },
+  { "transcript", mu_c_bool,
     NULL, mu_offsetof (struct mu_srv_config, transcript), NULL,
     N_("Log the session transcript.") },
-  { "timeout", mu_cfg_time,
+  { "timeout", mu_c_time,
     NULL, mu_offsetof (struct mu_srv_config, timeout), NULL,
     N_("Set idle timeout.") },
   { "acl", mu_cfg_section,
@@ -924,8 +957,33 @@ static struct mu_cfg_param server_cfg_param[] = {
   { NULL }
 };
 
+static int
+_cb_backlog (void *data, mu_config_value_t *val)
+{
+  mu_ip_server_t *psrv = data;
+  int backlog;
+  
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
+  if (sscanf (val->v.string, "%d", &backlog) != 1 || backlog <= 0
+      || mu_tcp_server_set_backlog (*psrv, backlog))
+    {
+      mu_error (_("invalid argument"));
+      return 1;
+    }
+  return 0;
+}
+
+static struct mu_cfg_param server_tcp_param[] = {
+  { "backlog", mu_cfg_callback,
+    NULL, mu_offsetof (struct mu_srv_config, tcpsrv), _cb_backlog,
+    N_("Size of the queue of pending connections"),
+    N_("number") },
+  { NULL }
+};
+
 void
-mu_m_server_cfg_init (struct mu_cfg_param *app_param)
+mu_m_server_cfg_init (mu_m_server_t srv, struct mu_cfg_param *app_param)
 {
   struct mu_cfg_section *section;
   if (mu_create_canned_section ("server", &section) == 0)
@@ -933,6 +991,8 @@ mu_m_server_cfg_init (struct mu_cfg_param *app_param)
       section->parser = server_section_parser;
       section->label = N_("ipaddr[:port]");
       mu_cfg_section_add_params (section, server_cfg_param);
+      if (srv->deftype == MU_IP_TCP)
+	mu_cfg_section_add_params (section, server_tcp_param);
       if (app_param)
 	mu_cfg_section_add_params (section, app_param);
     }
