@@ -1,6 +1,6 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2006, 2007, 2009 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2004, 2006-2007, 2009-2012, 2014-2018 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,10 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA
-*/
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #include <mailutils/cpp/stream.h>
 
@@ -55,7 +53,7 @@ Stream :: ~Stream ()
     {
       close ();
       if (this->stm)
-	mu_stream_destroy (&stm, NULL);
+	mu_stream_destroy (&stm);
     }
 }
 
@@ -108,9 +106,9 @@ Stream :: wait (int flags)
 }
 
 int
-Stream :: read (char* rbuf, size_t size, off_t offset)
+Stream :: read (void* rbuf, size_t size)
 {
-  int status = mu_stream_read (stm, rbuf, size, offset, &read_count);
+  int status = mu_stream_read (stm, rbuf, size, &read_count);
   if (status == EAGAIN)
     throw Stream::EAgain ("Stream::read", status);
   else if (status)
@@ -119,10 +117,9 @@ Stream :: read (char* rbuf, size_t size, off_t offset)
 }
 
 int
-Stream :: write (const std::string& wbuf, size_t size, off_t offset)
+Stream :: write (void* wbuf, size_t size)
 {
-  int status = mu_stream_write (stm, wbuf.c_str (), size, offset,
-				&write_count);
+  int status = mu_stream_write (stm, wbuf, size, &write_count);
   if (status == EAGAIN)
     throw Stream::EAgain ("Stream::write", status);
   else if (status)
@@ -131,9 +128,20 @@ Stream :: write (const std::string& wbuf, size_t size, off_t offset)
 }
 
 int
-Stream :: readline (char* rbuf, size_t size, off_t offset)
+Stream :: write (const std::string& wbuf, size_t size)
 {
-  int status = mu_stream_readline (stm, rbuf, size, offset, &read_count);
+  int status = mu_stream_write (stm, wbuf.c_str (), size, &write_count);
+  if (status == EAGAIN)
+    throw Stream::EAgain ("Stream::write", status);
+  else if (status)
+    throw Exception ("Stream::write", status);
+  return status;
+}
+
+int
+Stream :: readline (char* rbuf, size_t size)
+{
+  int status = mu_stream_readline (stm, rbuf, size, &read_count);
   if (status == EAGAIN)
     throw Stream::EAgain ("Stream::readline", status);
   else if (status)
@@ -141,22 +149,6 @@ Stream :: readline (char* rbuf, size_t size, off_t offset)
   return status;
 }
 
-void
-Stream :: sequential_readline (char* rbuf, size_t size)
-{
-  int status = mu_stream_sequential_readline (stm, rbuf, size, &read_count);
-  if (status)
-    throw Exception ("Stream::sequential_readline", status);
-}
-
-void
-Stream :: sequential_write (const std::string& wbuf, size_t size)
-{
-  int status = mu_stream_sequential_write (stm, wbuf.c_str (), size);
-  if (status)
-    throw Exception ("Stream::sequential_write", status);
-}
-
 void
 Stream :: flush ()
 {
@@ -170,7 +162,7 @@ namespace mailutils
   Stream&
   operator << (Stream& stm, const std::string& wbuf)
   {
-    stm.write (wbuf, wbuf.length (), 0);
+    stm.write (wbuf, wbuf.length ());
     return stm;
   }
 
@@ -178,7 +170,7 @@ namespace mailutils
   operator >> (Stream& stm, std::string& rbuf)
   {
     char tmp[1024];
-    stm.read (tmp, sizeof (tmp), 0);
+    stm.read (tmp, sizeof (tmp));
     rbuf = std::string (tmp);
     return stm;
   }
@@ -210,20 +202,22 @@ FileStream :: FileStream (const std::string& filename, int flags)
 // StdioStream
 //
 
-StdioStream :: StdioStream (FILE* fp, int flags)
+StdioStream :: StdioStream (int fd, int flags)
 {
-  int status = mu_stdio_stream_create (&stm, fp, flags);
+  int status = mu_stdio_stream_create (&stm, fd, flags);
   if (status)
     throw Exception ("StdioStream::StdioStream", status);
 }
 
 //
 // ProgStream
-//
+/* FIXME: This must be revised using the real prog_stream interface, instead
+   of the command_stream
+*/
 
 ProgStream :: ProgStream (const std::string& progname, int flags)
 {
-  int status = mu_prog_stream_create (&stm, progname.c_str (), flags);
+  int status = mu_command_stream_create (&stm, progname.c_str (), flags);
   if (status)
     throw Exception ("ProgStream::ProgStream", status);
 }
@@ -235,20 +229,28 @@ ProgStream :: ProgStream (const std::string& progname, int flags)
 FilterProgStream :: FilterProgStream (const std::string& progname,
 				      Stream& input)
 {
-  int status = mu_filter_prog_stream_create (&stm, progname.c_str (),
-					     input.stm);
-  this->input = new Stream (input);
+  struct mu_prog_hints hints;
+  struct mu_wordsplit ws;
+  int status;
+
+  if (mu_wordsplit (progname.c_str (), &ws, MU_WRDSF_DEFFLAGS))
+    throw Exception ("FilterProgStream::FilterProgStream", errno); 
+  
+  hints.mu_prog_input = input.stm;
+  status = mu_prog_stream_create (&stm, ws.ws_wordv[0],
+				  ws.ws_wordc, ws.ws_wordv,
+				  MU_PROG_HINT_INPUT,
+				  &hints,
+				  MU_STREAM_READ);
+  mu_wordsplit_free (&ws);
   if (status)
     throw Exception ("FilterProgStream::FilterProgStream", status);
+  this->input = new Stream (input);
 }
 
 FilterProgStream :: FilterProgStream (const std::string& progname,
 				      Stream* input)
 {
-  int status = mu_filter_prog_stream_create (&stm, progname.c_str (),
-					     input->stm);
-  this->input = new Stream (*input);
-  if (status)
-    throw Exception ("FilterProgStream::FilterProgStream", status);
+  FilterProgStream (progname, input);
 }
 
