@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005-2007, 2010-2012, 2014-2017 Free
+   Copyright (C) 1999-2001, 2005-2007, 2010-2012, 2014-2018 Free
    Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -33,21 +33,13 @@ struct mu_body
 #define BUF_SIZE 64
 
 /* SMOB functions: */
-static SCM
-mu_scm_body_mark (SCM body_smob)
-{
-  struct mu_body *mbp = (struct mu_body *) SCM_CDR (body_smob);
-  return mbp->msg;
-}
-
-static scm_sizet
+static size_t
 mu_scm_body_free (SCM body_smob)
 {
   struct mu_body *mbp = (struct mu_body *) SCM_CDR (body_smob);
   if (mbp->buffer)
     free (mbp->buffer);
   mu_stream_unref (mbp->stream);
-  free (mbp);
   return 0;
 }
 
@@ -56,16 +48,14 @@ mu_scm_body_print (SCM body_smob, SCM port, scm_print_state * pstate)
 {
   struct mu_body *mbp = (struct mu_body *) SCM_CDR (body_smob);
   size_t b_size = 0, b_lines = 0;
-  char buffer[512];
 
   mu_body_size (mbp->body, &b_size);
   mu_body_lines (mbp->body, &b_lines);
 
   scm_puts ("#<body ", port);
-  snprintf (buffer, sizeof (buffer), "%3lu %-5lu",
-	    (unsigned long) b_lines, (unsigned long) b_size);
-  scm_puts (buffer, port);
-
+  scm_intprint (b_lines, 10, port);
+  scm_putc (' ', port);
+  scm_intprint (b_size, 10, port);
   scm_puts (">", port);
   return 1;
 }
@@ -148,7 +138,8 @@ SCM_DEFINE_PUBLIC (scm_mu_body_read_line, "mu-body-read-line", 1, 0, 0,
 
 SCM_DEFINE_PUBLIC (scm_mu_body_write, "mu-body-write", 2, 0, 0,
 	    (SCM body, SCM text),
-	    "Append @var{text} to message @var{body}.")
+"Append @var{text} to message @var{body}. The use of this function is not\n"
+"recommended. Please, use message ports instead (@pxref{mu-message-get-port}).\n")
 #define FUNC_NAME s_scm_mu_body_write
 {
   char *ptr;
@@ -165,16 +156,20 @@ SCM_DEFINE_PUBLIC (scm_mu_body_write, "mu-body-write", 2, 0, 0,
       status = mu_body_get_streamref (mbp->body, &mbp->stream);
       if (status)
 	mu_scm_error (FUNC_NAME, status,
-		      "Cannot get body stream", SCM_BOOL_F);
+		      "mu_body_get_streamref", SCM_BOOL_F);
+      status = mu_stream_seek (mbp->stream, 0, SEEK_END, NULL);
+      if (status)
+	mu_scm_error (FUNC_NAME, status,
+		      "mu_stream_seek", SCM_BOOL_F);
     }
-
+  
   ptr = scm_to_locale_string (text);
   len = strlen (ptr);
   status = mu_stream_write (mbp->stream, ptr, len, NULL);
   free (ptr);
   if (status)
     mu_scm_error (FUNC_NAME, status,
-		  "Error writing to stream", SCM_BOOL_F);
+		  "mu_stream_write", SCM_BOOL_F);
   return SCM_BOOL_T;
 }
 #undef FUNC_NAME
@@ -184,7 +179,6 @@ void
 mu_scm_body_init ()
 {
   body_tag = scm_make_smob_type ("body", sizeof (struct mu_body));
-  scm_set_smob_mark (body_tag, mu_scm_body_mark);
   scm_set_smob_free (body_tag, mu_scm_body_free);
   scm_set_smob_print (body_tag, mu_scm_body_print);
 
