@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003-2004, 2007, 2010-2012 Free Software Foundation,
-   Inc.
+   Copyright (C) 2003-2004, 2007, 2010-2012, 2014-2018 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
