@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2002, 2005, 2007, 2010-2012, 2014-2018 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -23,15 +22,15 @@
  */
 
 int
-mail_inc (int argc ARG_UNUSED, char **argv ARG_UNUSED)
+mail_inc (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
 {
-  if (!mailbox_is_updated (mbox))
+  if (!mu_mailbox_is_updated (mbox))
     {
-      mailbox_messages_count (mbox, &total);
-      fprintf (ofile, _("New mail has arrived.\n"));
+      mu_mailbox_messages_count (mbox, &total);
+      mu_printf (_("New mail has arrived.\n"));
     }
   else
-    fprintf (ofile, _("No new mail for %s\n"), mail_whoami());
+    mu_printf (_("No new mail for %s\n"), mail_whoami ());
 
   return 0;
 }
