@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2003, 2005, 2007, 2010-2012 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999, 2001-2003, 2005, 2007, 2010-2012, 2014-2018 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ mail_mbox_close ()
   if (!mbox)
     return 0;
 
-  if (mailvar_get (NULL, "readonly", mailvar_type_boolean, 0))
+  if (!mailvar_is_true ("readonly"))
     {
       if (mail_mbox_commit ())
 	return 1;
@@ -50,10 +50,10 @@ mail_mbox_close ()
   mu_mailbox_get_url (mbox, &url);
   mu_mailbox_messages_count (mbox, &held_count);
   mu_printf (
-           ngettext ("Held %d message in %s\n",
-                     "Held %d messages in %s\n",
+           ngettext ("Held %lu message in %s\n",
+                     "Held %lu messages in %s\n",
                      held_count),
-           held_count, util_url_to_string (url));
+           (unsigned long) held_count, util_url_to_string (url));
   mu_mailbox_close (mbox);
   mu_mailbox_destroy (&mbox);
   return 0;
@@ -67,8 +67,8 @@ mail_mbox_commit ()
   int saved_count = 0;
   mu_message_t msg;
   mu_attribute_t attr;
-  int keepsave = mailvar_get (NULL, "keepsave", mailvar_type_boolean, 0) == 0;
-  int hold = mailvar_get (NULL, "hold", mailvar_type_boolean, 0) == 0;
+  int keepsave = mailvar_is_true ("keepsave");
+  int hold = mailvar_is_true ("hold");
   mu_url_t url;
   int is_user_mbox;
 
