@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005-2012, 2014-2016 Free Software Foundation,
+   Copyright (C) 2003, 2005-2012, 2014-2018 Free Software Foundation,
    Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -127,13 +127,17 @@ copy_message (mu_mailbox_t mbox, size_t n, const char *file)
   return rc;
 }
 
+static struct mh_optinit optinit[] = {
+  { "draftfolder", "Draft-Folder" },
+  { "whatnowproc", "whatnowproc" },
+  { NULL }
+};
+
 int
 main (int argc, char **argv)
 {
-  draftfolder = mh_global_profile_get ("Draft-Folder", NULL);
-  whatnowproc = mh_global_profile_get ("whatnowproc", NULL);
+  mh_getopt_ext (&argc, &argv, options, 0, optinit, args_doc, prog_doc, NULL);
 
-  mh_getopt (&argc, &argv, options, 0, args_doc, prog_doc, NULL);
   if (use_draft)
     draftmessage = "cur";
   if (!formfile)
@@ -189,7 +193,8 @@ main (int argc, char **argv)
 		  mu_error (_("only one message at a time!"));
 		  return 1;
 		}
-	      draftmessage = (char*) mu_umaxtostr (0, mh_msgset_first_uid (msgset));
+	      draftmessage =
+		(char*) mu_umaxtostr (0, mh_msgset_first (msgset, RET_UID));
 	      mu_msgset_free (msgset);
 	      mu_mailbox_destroy (&mbox);
 	    }
@@ -213,7 +218,7 @@ main (int argc, char **argv)
 	  return 1;
 	}
       unlink (wh_env.file);
-      copy_message (mbox, mh_msgset_first (msgset), wh_env.file);
+      copy_message (mbox, mh_msgset_first (msgset, RET_MSGNO), wh_env.file);
       mu_mailbox_destroy (&mbox);
       mu_msgset_free (msgset);
     }
