@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 2.5.1.  */
+/* A Bison parser, made by GNU Bison 2.7.  */
 
 /* Bison implementation for Yacc-like parsers in C
    
@@ -44,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.5.1"
+#define YYBISON_VERSION "2.7"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -58,18 +58,15 @@
 /* Pull parsers.  */
 #define YYPULL 1
 
-/* Using locations.  */
-#define YYLSP_NEEDED 0
 
 
 
 /* Copy the first part of user declarations.  */
-
-/* Line 268 of yacc.c  */
+/* Line 371 of yacc.c  */
 #line 1 "mh_fmtgram.y"
 
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2004, 2007, 2009-2012, 2014-2017 Free
+   Copyright (C) 1999-2002, 2004, 2007, 2009-2012, 2014-2018 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -104,17 +101,15 @@ enum context
     ctx_func,   /* after (func */
   };
 
-static enum context ctx_stack[512];
+static enum context *ctx_stack;
 size_t ctx_tos;
-
+size_t ctx_max;
+ 
 static inline void
 ctx_push (enum context ctx)
 {
-  if (ctx_tos == MU_ARRAY_SIZE (ctx_stack))
-    {
-      fmt_yyerror ("context nesting level too deep");
-      exit (1);
-    }
+  if (ctx_tos == ctx_max)
+    ctx_stack = mu_2nrealloc (ctx_stack, &ctx_max, sizeof (ctx_stack[0]));
   ctx_stack[ctx_tos++] = ctx;
 }
 
@@ -185,9 +180,8 @@ static void codegen (mh_format_t *fmt, int tree);
 static struct node *typecast (struct node *node, enum mh_type type);
  
 
-
-/* Line 268 of yacc.c  */
-#line 191 "mh_fmtgram.c"
+/* Line 371 of yacc.c  */
+#line 185 "mh_fmtgram.c"
 
 # ifndef YY_NULL
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -197,11 +191,6 @@ static struct node *typecast (struct node *node, enum mh_type type);
 #  endif
 # endif
 
-/* Enabling traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 1
-#endif
-
 /* Enabling verbose error messages.  */
 #ifdef YYERROR_VERBOSE
 # undef YYERROR_VERBOSE
@@ -210,11 +199,14 @@ static struct node *typecast (struct node *node, enum mh_type type);
 # define YYERROR_VERBOSE 1
 #endif
 
-/* Enabling the token table.  */
-#ifndef YYTOKEN_TABLE
-# define YYTOKEN_TABLE 0
-#endif
 
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 1
+#endif
+#if YYDEBUG
+extern int fmt_yydebug;
+#endif
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -252,13 +244,11 @@ static struct node *typecast (struct node *node, enum mh_type type);
 
 
 
-
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-
-/* Line 295 of yacc.c  */
-#line 120 "mh_fmtgram.y"
+/* Line 387 of yacc.c  */
+#line 118 "mh_fmtgram.y"
 
   char *str;
   char const *mesg;
@@ -279,21 +269,36 @@ typedef union YYSTYPE
   } arg;
 
 
-
-/* Line 295 of yacc.c  */
-#line 285 "mh_fmtgram.c"
+/* Line 387 of yacc.c  */
+#line 274 "mh_fmtgram.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
+extern YYSTYPE fmt_yylval;
+
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int fmt_yyparse (void *YYPARSE_PARAM);
+#else
+int fmt_yyparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int fmt_yyparse (void);
+#else
+int fmt_yyparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
+
+
 
 /* Copy the second part of user declarations.  */
 
-
-/* Line 345 of yacc.c  */
-#line 297 "mh_fmtgram.c"
+/* Line 390 of yacc.c  */
+#line 302 "mh_fmtgram.c"
 
 #ifdef short
 # undef short
@@ -346,24 +351,24 @@ typedef short int yytype_int16;
 # if defined YYENABLE_NLS && YYENABLE_NLS
 #  if ENABLE_NLS
 #   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
-#   define YY_(msgid) dgettext ("bison-runtime", msgid)
+#   define YY_(Msgid) dgettext ("bison-runtime", Msgid)
 #  endif
 # endif
 # ifndef YY_
-#  define YY_(msgid) msgid
+#  define YY_(Msgid) Msgid
 # endif
 #endif
 
 /* Suppress unused-variable warnings by "using" E.  */
 #if ! defined lint || defined __GNUC__
-# define YYUSE(e) ((void) (e))
+# define YYUSE(E) ((void) (E))
 #else
-# define YYUSE(e) /* empty */
+# define YYUSE(E) /* empty */
 #endif
 
 /* Identity function, used to suppress warnings about constant conditions.  */
 #ifndef lint
-# define YYID(n) (n)
+# define YYID(N) (N)
 #else
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
@@ -590,14 +595,14 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   159,   159,   165,   169,   178,   185,   188,   189,   202,
-     203,   206,   218,   318,   321,   324,   331,   334,   352,   359,
-     369,   372,   375,   381,   387,   394,   402,   403,   407,   410,
-     411,   417,   425,   437,   445
+       0,   157,   157,   163,   167,   176,   183,   186,   187,   200,
+     201,   204,   216,   316,   319,   322,   329,   332,   350,   357,
+     367,   370,   373,   379,   385,   392,   400,   401,   405,   408,
+     409,   415,   423,   435,   443
 };
 #endif
 
-#if YYDEBUG || YYERROR_VERBOSE || YYTOKEN_TABLE
+#if YYDEBUG || YYERROR_VERBOSE || 1
 /* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const yytname[] =
@@ -691,10 +696,10 @@ static const yytype_int8 fmt_yytable[] =
        5,    24,     0,    16,    17,    39,    40
 };
 
-#define yypact_value_is_default(fmt_yystate) \
-  ((fmt_yystate) == (-29))
+#define yypact_value_is_default(Yystate) \
+  (!!((Yystate) == (-29)))
 
-#define yytable_value_is_error(yytable_value) \
+#define yytable_value_is_error(Yytable_value) \
   YYID (0)
 
 static const yytype_int8 fmt_yycheck[] =
@@ -760,46 +765,18 @@ do                                                              \
     }								\
 while (YYID (0))
 
-
+/* Error token number */
 #define YYTERROR	1
 #define YYERRCODE	256
 
 
-/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
-   If N is 0, then set CURRENT to the empty location which ends
-   the previous symbol: RHS[0] (always defined).  */
-
-#define YYRHSLOC(Rhs, K) ((Rhs)[K])
-#ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)				\
-    do									\
-      if (YYID (N))                                                    \
-	{								\
-	  (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;	\
-	  (Current).first_column = YYRHSLOC (Rhs, 1).first_column;	\
-	  (Current).last_line    = YYRHSLOC (Rhs, N).last_line;		\
-	  (Current).last_column  = YYRHSLOC (Rhs, N).last_column;	\
-	}								\
-      else								\
-	{								\
-	  (Current).first_line   = (Current).last_line   =		\
-	    YYRHSLOC (Rhs, 0).last_line;				\
-	  (Current).first_column = (Current).last_column =		\
-	    YYRHSLOC (Rhs, 0).last_column;				\
-	}								\
-    while (YYID (0))
-#endif
-
-
 /* This macro is provided for backward compatibility. */
-
 #ifndef YY_LOCATION_PRINT
 # define YY_LOCATION_PRINT(File, Loc) ((void) 0)
 #endif
 
 
 /* YYLEX -- calling `fmt_yylex' with the right arguments.  */
-
 #ifdef YYLEX_PARAM
 # define YYLEX fmt_yylex (YYLEX_PARAM)
 #else
@@ -862,7 +839,7 @@ yy_symbol_value_print (yyoutput, yytype, yyvaluep)
   switch (yytype)
     {
       default:
-	break;
+        break;
     }
 }
 
@@ -1104,7 +1081,6 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 {
   YYSIZE_T yysize0 = yytnamerr (YY_NULL, yytname[yytoken]);
   YYSIZE_T yysize = yysize0;
-  YYSIZE_T yysize1;
   enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
   /* Internationalized format string. */
   const char *yyformat = YY_NULL;
@@ -1167,11 +1143,13 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
                     break;
                   }
                 yyarg[yycount++] = yytname[yyx];
-                yysize1 = yysize + yytnamerr (YY_NULL, yytname[yyx]);
-                if (! (yysize <= yysize1
-                       && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
-                  return 2;
-                yysize = yysize1;
+                {
+                  YYSIZE_T yysize1 = yysize + yytnamerr (YY_NULL, yytname[yyx]);
+                  if (! (yysize <= yysize1
+                         && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+                    return 2;
+                  yysize = yysize1;
+                }
               }
         }
     }
@@ -1191,10 +1169,12 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 # undef YYCASE_
     }
 
-  yysize1 = yysize + yystrlen (yyformat);
-  if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
-    return 2;
-  yysize = yysize1;
+  {
+    YYSIZE_T yysize1 = yysize + yystrlen (yyformat);
+    if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+      return 2;
+    yysize = yysize1;
+  }
 
   if (*yymsg_alloc < yysize)
     {
@@ -1254,32 +1234,27 @@ yydestruct (yymsg, yytype, yyvaluep)
     {
 
       default:
-	break;
+        break;
     }
 }
 
 
-/* Prevent warnings from -Wmissing-prototypes.  */
-#ifdef YYPARSE_PARAM
-#if defined __STDC__ || defined __cplusplus
-int fmt_yyparse (void *YYPARSE_PARAM);
-#else
-int fmt_yyparse ();
-#endif
-#else /* ! YYPARSE_PARAM */
-#if defined __STDC__ || defined __cplusplus
-int fmt_yyparse (void);
-#else
-int fmt_yyparse ();
-#endif
-#endif /* ! YYPARSE_PARAM */
 
 
 /* The lookahead symbol.  */
 int fmt_yychar;
 
+
+#ifndef YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
+# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
+# define YY_IGNORE_MAYBE_UNINITIALIZED_END
+#endif
+#ifndef YY_INITIAL_VALUE
+# define YY_INITIAL_VALUE(Value) /* Nothing. */
+#endif
+
 /* The semantic value of the lookahead symbol.  */
-YYSTYPE fmt_yylval;
+YYSTYPE fmt_yylval YY_INITIAL_VALUE(yyval_default);
 
 /* Number of syntax errors so far.  */
 int fmt_yynerrs;
@@ -1337,7 +1312,7 @@ fmt_yyparse ()
   int yyn;
   int yyresult;
   /* Lookahead token as an internal (translated) token number.  */
-  int yytoken;
+  int yytoken = 0;
   /* The variables used to return semantic value and location from the
      action routines.  */
   YYSTYPE fmt_yyval;
@@ -1355,9 +1330,8 @@ fmt_yyparse ()
      Keep to zero when no symbol should be popped.  */
   int fmt_yylen = 0;
 
-  yytoken = 0;
-  yyss = yyssa;
-  yyvs = yyvsa;
+  yyssp = yyss = yyssa;
+  yyvsp = yyvs = yyvsa;
   yystacksize = YYINITDEPTH;
 
   YYDPRINTF ((stderr, "Starting parse\n"));
@@ -1366,14 +1340,6 @@ fmt_yyparse ()
   yyerrstatus = 0;
   fmt_yynerrs = 0;
   fmt_yychar = YYEMPTY; /* Cause a token to be read.  */
-
-  /* Initialize stack pointers.
-     Waste one element of value and location stack
-     so that they stay on the same level as the state stack.
-     The wasted elements are never initialized.  */
-  yyssp = yyss;
-  yyvsp = yyvs;
-
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -1514,7 +1480,9 @@ yybackup:
   fmt_yychar = YYEMPTY;
 
   fmt_yystate = yyn;
+  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   *++yyvsp = fmt_yylval;
+  YY_IGNORE_MAYBE_UNINITIALIZED_END
 
   goto yynewstate;
 
@@ -1551,27 +1519,24 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-
-/* Line 1810 of yacc.c  */
-#line 160 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 158 "mh_fmtgram.y"
     {
 	      parse_tree = (yyvsp[(1) - (1)].nodelist).head;
 	    }
     break;
 
   case 3:
-
-/* Line 1810 of yacc.c  */
-#line 166 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 164 "mh_fmtgram.y"
     {
 	      (fmt_yyval.nodelist).head = (fmt_yyval.nodelist).tail = (yyvsp[(1) - (1)].nodeptr);
 	    }
     break;
 
   case 4:
-
-/* Line 1810 of yacc.c  */
-#line 170 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 168 "mh_fmtgram.y"
     {
 	      (yyvsp[(2) - (2)].nodeptr)->prev = (yyvsp[(1) - (2)].nodelist).tail;
 	      (yyvsp[(1) - (2)].nodelist).tail->next = (yyvsp[(2) - (2)].nodeptr);
@@ -1581,9 +1546,8 @@ yyreduce:
     break;
 
   case 5:
-
-/* Line 1810 of yacc.c  */
-#line 179 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 177 "mh_fmtgram.y"
     {
 	      struct node *n = new_node (fmtnode_literal, mhtype_str);
 	      n->v.str = (yyvsp[(1) - (1)].str);
@@ -1593,9 +1557,8 @@ yyreduce:
     break;
 
   case 8:
-
-/* Line 1810 of yacc.c  */
-#line 190 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 188 "mh_fmtgram.y"
     {
 	      if ((yyvsp[(2) - (2)].nodeptr)->printflag & MHA_NOPRINT)
 		(fmt_yyval.nodeptr) = (yyvsp[(2) - (2)].nodeptr);
@@ -1609,9 +1572,8 @@ yyreduce:
     break;
 
   case 11:
-
-/* Line 1810 of yacc.c  */
-#line 207 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 205 "mh_fmtgram.y"
     {
 	      if (mu_c_strcasecmp ((yyvsp[(1) - (1)].str), "body") == 0)
 		(fmt_yyval.nodeptr) = new_node (fmtnode_body, mhtype_str);
@@ -1624,9 +1586,8 @@ yyreduce:
     break;
 
   case 12:
-
-/* Line 1810 of yacc.c  */
-#line 219 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 217 "mh_fmtgram.y"
     {
 	      struct node *arg;
 
@@ -1726,36 +1687,32 @@ yyreduce:
     break;
 
   case 13:
-
-/* Line 1810 of yacc.c  */
-#line 318 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 316 "mh_fmtgram.y"
     {
 	      (fmt_yyval.fmtspec) = 0;
 	    }
     break;
 
   case 15:
-
-/* Line 1810 of yacc.c  */
-#line 325 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 323 "mh_fmtgram.y"
     {
 	      ctx_push (ctx_func);
 	    }
     break;
 
   case 16:
-
-/* Line 1810 of yacc.c  */
-#line 331 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 329 "mh_fmtgram.y"
     {
 	      (fmt_yyval.nodeptr) = NULL;
 	    }
     break;
 
   case 17:
-
-/* Line 1810 of yacc.c  */
-#line 335 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 333 "mh_fmtgram.y"
     {
 	      switch ((yyvsp[(1) - (1)].arg).type)
 		{
@@ -1776,18 +1733,16 @@ yyreduce:
     break;
 
   case 18:
-
-/* Line 1810 of yacc.c  */
-#line 353 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 351 "mh_fmtgram.y"
     {
 	      (fmt_yyval.nodeptr) = printelim ((yyvsp[(1) - (1)].nodeptr));
 	    }
     break;
 
   case 19:
-
-/* Line 1810 of yacc.c  */
-#line 360 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 358 "mh_fmtgram.y"
     {
 	      (fmt_yyval.nodeptr) = new_node(fmtnode_cntl, mhtype_num);
 	      (fmt_yyval.nodeptr)->v.cntl.cond = (yyvsp[(2) - (5)].nodeptr);
@@ -1797,36 +1752,32 @@ yyreduce:
     break;
 
   case 20:
-
-/* Line 1810 of yacc.c  */
-#line 369 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 367 "mh_fmtgram.y"
     {
 	      (fmt_yyval.nodelist).head = (fmt_yyval.nodelist).tail = NULL;
 	    }
     break;
 
   case 22:
-
-/* Line 1810 of yacc.c  */
-#line 376 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 374 "mh_fmtgram.y"
     {
 	      ctx_push (ctx_if);
 	    }
     break;
 
   case 23:
-
-/* Line 1810 of yacc.c  */
-#line 382 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 380 "mh_fmtgram.y"
     {
 	      ctx_pop ();
 	    }
     break;
 
   case 24:
-
-/* Line 1810 of yacc.c  */
-#line 388 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 386 "mh_fmtgram.y"
     {
 	      ctx_pop ();
 	      ctx_push (ctx_if);
@@ -1834,9 +1785,8 @@ yyreduce:
     break;
 
   case 25:
-
-/* Line 1810 of yacc.c  */
-#line 395 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 393 "mh_fmtgram.y"
     {
 	      ctx_pop ();
 	      ctx_push (ctx_expr);
@@ -1845,27 +1795,24 @@ yyreduce:
     break;
 
   case 28:
-
-/* Line 1810 of yacc.c  */
-#line 407 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 405 "mh_fmtgram.y"
     {
 	      (fmt_yyval.nodeptr) = NULL;
 	    }
     break;
 
   case 30:
-
-/* Line 1810 of yacc.c  */
-#line 412 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 410 "mh_fmtgram.y"
     {
 	      (fmt_yyval.nodeptr) = (yyvsp[(1) - (1)].nodelist).head;
 	    }
     break;
 
   case 31:
-
-/* Line 1810 of yacc.c  */
-#line 418 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 416 "mh_fmtgram.y"
     {
 	      struct node *np = new_node (fmtnode_cntl, mhtype_num);
 	      np->v.cntl.cond = (yyvsp[(2) - (3)].nodeptr);
@@ -1876,9 +1823,8 @@ yyreduce:
     break;
 
   case 32:
-
-/* Line 1810 of yacc.c  */
-#line 426 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 424 "mh_fmtgram.y"
     {
 	      struct node *np = new_node(fmtnode_cntl, mhtype_num);
 	      np->v.cntl.cond = (yyvsp[(3) - (4)].nodeptr);
@@ -1893,9 +1839,8 @@ yyreduce:
     break;
 
   case 33:
-
-/* Line 1810 of yacc.c  */
-#line 438 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 436 "mh_fmtgram.y"
     {
 	      (yyvsp[(1) - (2)].nodelist).tail->v.cntl.iffalse = (yyvsp[(2) - (2)].nodeptr);
 	      (yyvsp[(1) - (2)].nodelist).tail = (yyvsp[(2) - (2)].nodeptr);
@@ -1904,18 +1849,16 @@ yyreduce:
     break;
 
   case 34:
-
-/* Line 1810 of yacc.c  */
-#line 446 "mh_fmtgram.y"
+/* Line 1792 of yacc.c  */
+#line 444 "mh_fmtgram.y"
     {
 	      (fmt_yyval.nodeptr) = (yyvsp[(2) - (2)].nodelist).head;
 	    }
     break;
 
 
-
-/* Line 1810 of yacc.c  */
-#line 1919 "mh_fmtgram.c"
+/* Line 1792 of yacc.c  */
+#line 1862 "mh_fmtgram.c"
       default: break;
     }
   /* User semantic actions sometimes alter fmt_yychar, and that requires
@@ -2078,7 +2021,9 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
+  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   *++yyvsp = fmt_yylval;
+  YY_IGNORE_MAYBE_UNINITIALIZED_END
 
 
   /* Shift the error token.  */
@@ -2144,9 +2089,8 @@ yyreturn:
 }
 
 
-
-/* Line 2071 of yacc.c  */
-#line 451 "mh_fmtgram.y"
+/* Line 2055 of yacc.c  */
+#line 449 "mh_fmtgram.y"
 
 
 static char *start;
@@ -2485,14 +2429,21 @@ yylex_initial (void)
 int
 yylex_cond (void)
 {
-  switch (peek ())
+  while (1)
     {
-    case '(':
-      return token_function ();
-    case '{':
-      return token_component ();
-    default:
-      return bogus ("'(' or '{' expected");
+      switch (peek ())
+	{
+	case '(':
+	  return token_function ();
+	case '{':
+	  return token_component ();
+	case '\\':
+	  input ();
+	  if (input () == '\n')
+	    continue;
+	default:
+	  return bogus ("'(' or '{' expected");
+	}
     }
 }
 
@@ -2628,7 +2579,8 @@ format_parse (mh_format_t *fmtptr, char *format_str,
   start = tok_start = curp = format_str;
   mu_opool_create (&tokpool, MU_OPOOL_ENOMEMABRT);
 
-  ctx_tos = 0;
+  ctx_tos = ctx_max = 0;
+  ctx_stack = NULL;
   ctx_push (ctx_init);
   mu_linetrack_create (&trk, "input", 2);
   if (locus && locus->mu_file)
@@ -2643,6 +2595,7 @@ format_parse (mh_format_t *fmtptr, char *format_str,
 
   mu_locus_range_deinit (&fmt_yylloc);
   mu_linetrack_destroy (&trk);
+  free (ctx_stack);
   
   parse_tree = NULL;
   tokpool = NULL;
@@ -3256,6 +3209,41 @@ codegen_node (struct mh_format *fmt, struct node *node)
       {
 	long pc[2];
 	
+	/* Implementation of control escapes is a bit tricky. According to
+	   the spec:
+	   
+	     "[f]unction escapes write their return value in 'num' for
+	      functions returning integer or boolean values"
+
+	   That means that after "%<(gt 1024)" the value of 'num' would be
+	   1 or 0, depending on its value prior to entering the conditional.
+	   However this would defeat the purpose of the conditional itself,
+	   because then the following construct would be meaningless:
+
+	       %<(gt 1024)...%?(gt 512)...%|...%>
+
+	   Indeed, in MH implementation the value of 'num' propagates into
+	   the conditional expression, because any function escape serving
+	   as condition is evaluated in a separate context.
+
+	   To ensure this behavior, the virtual machine of GNU MH holds the
+	   value of the 'num' register on stack while evaluating the condition
+	   and restores it afterward.
+
+	   On the other hand, the spec says that:
+
+ 	     "[c]ontrol escapes return a boolean value, setting num to 1
+	     if the last explicit condition evaluated by a `%<'  or `%?'
+	     control succeeded, and 0 otherwise."
+
+	   To ensure this, the value on top of stack is exchanged with the
+	   value of the 'num' register upon entering the 'if' branch, and
+	   the tos value is popped into the 'num' upon leaving it. Any
+	   'else if' branches are handled the same way.
+
+	   Before leaving the 'else' branch, the 'num' is set to 0 explicitly.
+	*/
+	emit_opcode (fmt, mhop_pushn);
 	codegen_node (fmt, node->v.cntl.cond);
 	emit_opcode_typed (fmt, node->v.cntl.cond->datatype,
 			   mhop_brzn, mhop_brzs);
@@ -3263,8 +3251,10 @@ codegen_node (struct mh_format *fmt, struct node *node)
 	emit_instr (fmt, (mh_instr_t) NULL);
 	if (node->v.cntl.iftrue)
 	  {
+	    emit_opcode (fmt, mhop_xchgn);
 	    codegen_nodelist (fmt, node->v.cntl.iftrue);
 	  }
+	emit_opcode (fmt, mhop_popn);
 
 	if (node->v.cntl.iffalse)
 	  {
@@ -3273,7 +3263,14 @@ codegen_node (struct mh_format *fmt, struct node *node)
 	    emit_instr (fmt, (mh_instr_t) NULL);
 	
 	    fmt->prog[pc[0]].num = fmt->progcnt - pc[0];
+	    emit_opcode (fmt, mhop_popn);
 	    codegen_nodelist (fmt, node->v.cntl.iffalse);
+	    if (node->v.cntl.iffalse->nodetype != fmtnode_cntl)
+	      {
+		emit_opcode (fmt, mhop_setn);
+		emit_instr (fmt, (mh_instr_t) (long) R_REG);
+		emit_instr (fmt, (mh_instr_t) (long) 0);
+	      }
 	    fmt->prog[pc[1]].num = fmt->progcnt - pc[1];
 	  }
 	else
@@ -3338,4 +3335,3 @@ codegen (mh_format_t *fmtptr, int tree)
 }
 
 
-
