@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005-2012, 2014-2016 Free Software Foundation,
+   Copyright (C) 2003, 2005-2012, 2014-2018 Free Software Foundation,
    Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -461,7 +461,7 @@ mhn_compose_command (char *typestr, char *typeargs, int *flags, char *file)
     %F  %f, and stdout is not redirected
     %s  subtype */
 
-  mu_opool_create (&pool, 1);
+  mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
 
   p = mu_str_skip_class (str, MU_CTYPE_SPACE);
   
@@ -642,7 +642,7 @@ mhn_show_command (mu_message_t msg, msg_part_t part, int *flags,
     %s  subtype
     %d  content description */
 
-  mu_opool_create (&pool, 1);
+  mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
 
   p = mu_str_skip_class (str, MU_CTYPE_SPACE);
   
@@ -814,7 +814,7 @@ mhn_store_command (mu_message_t msg, msg_part_t part, const char *name,
      %p  part
      %s  subtype */
 
-  mu_opool_create (&pool, 1);
+  mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
   
   for (p = str; *p; p++)
     {
@@ -1172,7 +1172,7 @@ list_handler (mu_message_t msg, msg_part_t part, char *type, char *encoding,
       get_extbody_params (msg, &content_type, &content_descr);
 	
       printf ("          ");
-      printf ("%-25s", content_type ? content_type : "");
+      printf ("%-25s", mu_prstr (content_type));
       if (content_descr)
 	printf ("       %s", content_descr);
       printf ("\n");
@@ -1204,7 +1204,7 @@ list_iterator (size_t num, mu_message_t msg, void *data)
 }
 
 int
-mhn_list ()
+mhn_list (void)
 {
   int rc; 
 
@@ -1329,7 +1329,7 @@ sigint (int sig)
 }
 
 static int
-mhn_pause ()
+mhn_pause (void)
 {
   char c;
   int rc = 0;
@@ -1438,7 +1438,7 @@ show_iterator (size_t num, mu_message_t msg, void *data)
 }
 
 int
-mhn_show ()
+mhn_show (void)
 {
   
   int rc;
@@ -1680,7 +1680,7 @@ store_iterator (size_t num, mu_message_t msg, void *data)
 }
 
 int
-mhn_store ()
+mhn_store (void)
 {
   int rc = 0;
   
@@ -1744,7 +1744,7 @@ parse_brace (char **pval, char **cmd, int c, struct compose_env *env)
   return 0;
 }
 
-#define isdelim(c) (mu_isspace (c) || strchr (";<[(", c))
+#define isdelim(c) (c == 0 || mu_isspace (c) || strchr (";<[(", c))
 #define skipws(ptr) (ptr) = mu_str_skip_class (ptr, MU_CTYPE_SPACE)
 
 int
@@ -1754,7 +1754,7 @@ parse_content_type (struct compose_env *env,
   int status = 0, stop = 0;
   char *rest = *prest;
   char *comment = NULL;
-
+  
   while (stop == 0 && status == 0 && *rest)
     {
       skipws (rest);
@@ -1823,12 +1823,29 @@ parse_content_type (struct compose_env *env,
 	  rest++;
 	  mu_opool_append_char (pool, '=');
 	  skipws (rest);
-	  for (; *rest; rest++)
+	  if (*rest == '"')
 	    {
-	      if (isdelim (*rest))
-		break;
 	      mu_opool_append_char (pool, *rest);
+	      rest++;
+	      while (*rest != '"')
+		{
+		  if (*rest == '\\')
+		    {
+		      mu_opool_append_char (pool, *rest);
+		      if (rest[1])
+			rest++;
+		      else
+			break;
+		    }
+		  mu_opool_append_char (pool, *rest);
+		  rest++;
+		}
+	      mu_opool_append_char (pool, *rest);
+	      rest++;
 	    }
+	  else
+	    for (; !isdelim (*rest); rest++)
+	      mu_opool_append_char (pool, *rest);
 	  break;
 	    
 	default:
@@ -1867,7 +1884,7 @@ parse_type_command (char **pcmd, struct compose_env *env, mu_header_t hdr)
   char *rest = *pcmd;
   
   skipws (rest);
-  for (sp = rest; *sp && !isdelim (*sp); sp++)
+  for (sp = rest; !isdelim (*sp); sp++)
     ;
   c = *sp;
   *sp = 0;
@@ -1883,7 +1900,7 @@ parse_type_command (char **pcmd, struct compose_env *env, mu_header_t hdr)
       return 1;
     }
 
-  mu_opool_create (&pool, 1);
+  mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
 
   mu_opool_appendz (pool, type);
   mu_opool_append_char (pool, '/');
@@ -1927,7 +1944,7 @@ copy_header (mu_message_t msg, mu_header_t out)
 	  mu_header_sget_field_value (hdr, i, &value))
 	continue;
 
-      mu_header_set_value (out, name, value, 0);
+      mu_header_append (out, name, value);
     }
 }
 
@@ -1992,7 +2009,7 @@ finish_text_msg (struct compose_env *env, mu_message_t *msg, int ascii)
       mu_message_get_header (newmsg, &hdr);
       copy_header (*msg, hdr);
       mu_header_set_value (hdr, MU_HEADER_CONTENT_TRANSFER_ENCODING,
-			   "quoted-printable", 0);
+			   "quoted-printable", 1);
       
       mu_message_get_body (newmsg, &body);
       mu_body_get_streamref (body, &output);
@@ -2050,7 +2067,7 @@ edit_extern (char *cmd, struct compose_env *env, mu_message_t *msg, int level)
 
   mu_message_get_header (*msg, &hdr);
 
-  mu_opool_create (&pool, 1);
+  mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
   mu_opool_append (pool, EXTCONTENT, sizeof (EXTCONTENT) - 1);
   *--rest = ';'; /* FIXME */
   rc = parse_content_type (env, pool, &rest, &id, NULL);
@@ -2680,7 +2697,7 @@ mhn_header (mu_message_t msg, mu_message_t omsg)
 }
 
 int
-mhn_compose ()
+mhn_compose (void)
 {
   int rc;
   mu_mime_t mime = NULL;
@@ -2743,12 +2760,22 @@ mhn_compose ()
   
   /* Preserve the backup copy and replace the draft */
   unlink (backup);
-  rename (input_file, backup);
-  rename (name, input_file);
+
+  rc = mu_rename_file (input_file, backup, MU_COPY_OVERWRITE);
+  if (rc)
+    mu_error (_("can't rename %s to backup file %s: %s"),
+	      input_file, backup, mu_strerror (rc));
+  else
+    {
+      rc = mu_rename_file (name, input_file, 0);
+      if (rc)
+	mu_error (_("can't rename %s to %s: %s"),
+		  name, input_file, mu_strerror (rc));
+    }
 
   free (name);
   mu_mime_unref (mime); 
-  return 0;
+  return rc;
 }
 
 
@@ -2766,7 +2793,7 @@ main (int argc, char **argv)
     pause_option = 0;
 
   if (!compose_option && !list_option && !show_option && !store_option)
-    show_option = 1;
+    compose_option = 1;
   
   signal (SIGPIPE, SIG_IGN);
 
