@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005-2012 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005-2012, 2014-2018 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,90 +23,13 @@
 #include <stdarg.h>
 #include <pwd.h>
 
-static char doc[] = N_("GNU MH send")"\v"
-N_("Options marked with `*' are not yet implemented.\n\
-Use -help to obtain the list of traditional MH options.");
+static char prog_doc[] = N_("Send messages");
 static char args_doc[] = N_("FILE [FILE...]");
 
-/* GNU options */
-static struct argp_option options[] = {
-  {"alias",         ARG_ALIAS,         N_("FILE"), 0,
-   N_("specify additional alias file") },
-  {"draft",         ARG_DRAFT,         NULL, 0,
-   N_("use prepared draft") },
-  {"draftfolder",   ARG_DRAFTFOLDER,   N_("FOLDER"), 0,
-   N_("specify the folder for message drafts") },
-  {"draftmessage",  ARG_DRAFTMESSAGE,  NULL, 0,
-   N_("treat the arguments as a list of messages from the draftfolder") },
-  {"nodraftfolder", ARG_NODRAFTFOLDER, NULL, 0,
-   N_("undo the effect of the last --draftfolder option") },
-  {"filter",        ARG_FILTER,        N_("FILE"), 0,
-  N_("* use filter FILE to preprocess the body of the message") },
-  {"nofilter",      ARG_NOFILTER,      NULL, 0,
-   N_("* undo the effect of the last --filter option") },
-  {"format",        ARG_FORMAT,        N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("* reformat To: and Cc: addresses") },
-  {"noformat",      ARG_NOFORMAT,      NULL, OPTION_HIDDEN, "" },
-  {"forward",       ARG_FORWARD,       N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("* in case of failure forward the draft along with the failure notice to the sender") },
-  {"noforward",     ARG_NOFORWARD,     NULL, OPTION_HIDDEN, "" },
-  {"mime",          ARG_MIME,          N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("* use MIME encapsulation") },
-  {"nomime",        ARG_NOMIME,        NULL, OPTION_HIDDEN, "" },
-  {"msgid",         ARG_MSGID,         N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("add Message-ID: field") },
-  {"nomsgid",       ARG_NOMSGID,       NULL, OPTION_HIDDEN, ""},
-  {"push",          ARG_PUSH,          N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("run in the backround") },
-  {"nopush",        ARG_NOPUSH,        NULL, OPTION_HIDDEN, "" },
-  {"preserve",      ARG_PRESERVE,      N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("keep draft files") },
-  {"keep",          0, NULL, OPTION_ALIAS, NULL},
-  {"split",         ARG_SPLIT,         N_("SECONDS"), 0,
-   N_("split the draft into several partial messages and send them with SECONDS interval") },
-  {"chunksize",     ARG_CHUNKSIZE,     N_("NUMBER"), 0,
-   N_("set the size of chunk for --split (in bytes)") },
-  {"verbose",       ARG_VERBOSE,       N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("print the transcript of interactions with the transport system") },
-  {"noverbose",     ARG_NOVERBOSE,     NULL, OPTION_HIDDEN, "" },
-  {"watch",         ARG_WATCH,         N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("monitor the delivery of mail") },
-  {"nowatch",       ARG_NOWATCH,       NULL, OPTION_HIDDEN, "" },
-  {"width",         ARG_WIDTH,         N_("NUMBER"), 0,
-   N_("* make header fields no longer than NUMBER columns") },
-  { 0 }
-};
-
-/* Traditional MH options */
-struct mh_option mh_option[] = {
-  { "alias",         MH_OPT_ARG, "aliasfile" },
-  { "draft" },
-  { "draftfolder",   MH_OPT_ARG, "folder" },
-  { "draftmessage",  MH_OPT_ARG, "message" },
-  { "nodraftfolder" },
-  { "filter",        MH_OPT_ARG, "filterfile" },
-  { "nofilter" },
-  { "format",        MH_OPT_BOOL },
-  { "forward",       MH_OPT_BOOL },
-  { "mime",          MH_OPT_BOOL },
-  { "msgid",         MH_OPT_BOOL },
-  { "push",          MH_OPT_BOOL },
-  { "preserve",      MH_OPT_BOOL },
-  { "keep",          MH_OPT_BOOL },
-  { "split",         MH_OPT_ARG, "seconds" },
-  { "verbose",       MH_OPT_BOOL },
-  { "watch",         MH_OPT_BOOL },
-  { "width" },
-  { NULL }
-};
-
-static int use_draft;            /* Use the prepared draft */
 static const char *draftfolder;  /* Use this draft folder */
-static char *draftmessage = "cur";
-static int reformat_recipients;  /* --format option */
-static int forward_notice;       /* Forward the failure notice to the sender,
-				    --forward flag */
-static int mime_encaps;          /* Use MIME encapsulation */
+static int use_draftfolder = 1;
+static int use_draft;
+
 static int append_msgid;         /* Append Message-ID: header */
 static int background;           /* Operate in the background */
 
@@ -115,7 +39,6 @@ static unsigned long split_interval; /* Interval in seconds between sending two
 static size_t split_size = 76*632;   /* Size of split parts */
 static int verbose;              /* Produce verbose diagnostics */
 static int watch;                /* Watch the delivery process */
-static unsigned width = 76;      /* Maximum width of header fields */
 
 static int keep_files;           /* Keep draft files */
 
@@ -126,142 +49,116 @@ static int keep_files;           /* Keep draft files */
     watch_printf c;\
 } while (0)
 
-static error_t
-opt_handler (int key, char *arg, struct argp_state *state)
+static int add_file (char const *name);
+static void mesg_list_fixup (void);
+
+static void
+add_alias (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 {
-  char *p;
-  
-  switch (key)
-    {
-    case ARG_ALIAS:
-      mh_alias_read (arg, 1);
-      break;
-
-    case ARG_CHUNKSIZE:
-      split_size = strtoul (arg, &p, 10);
-      if (*p)
-	{
-	  argp_error (state, "%s: %s", arg, _("invalid number"));
-	  exit (1);
-	}
-      break;
-	
-    case ARG_DRAFT:
-      use_draft = 1;
-      break;
-	
-    case ARG_DRAFTFOLDER:
-      draftfolder = arg;
-      break;
-      
-    case ARG_NODRAFTFOLDER:
-      draftfolder = NULL;
-      break;
-      
-    case ARG_DRAFTMESSAGE:
-      draftmessage = arg;
-      break;
-      
-    case ARG_FILTER:
-      mh_opt_notimpl ("-filter");
-      break;
-
-    case ARG_NOFILTER:
-      mh_opt_notimpl ("-nofilter");
-      break;
- 
-    case ARG_FORMAT:
-      mh_opt_notimpl_warning ("-format"); 
-      reformat_recipients = is_true (arg);
-      break;
-      
-    case ARG_NOFORMAT:
-      mh_opt_notimpl_warning ("-noformat"); 
-      reformat_recipients = 0;
-      break;
-      
-    case ARG_FORWARD:
-      mh_opt_notimpl_warning ("-forward");
-      forward_notice = is_true (arg);
-      break;
-      
-    case ARG_NOFORWARD:
-      mh_opt_notimpl_warning ("-noforward");
-      forward_notice = 0;
-      break;
-      
-    case ARG_MIME:
-      mh_opt_notimpl_warning ("-mime");
-      mime_encaps = is_true (arg);
-      break;
-      
-    case ARG_NOMIME:
-      mh_opt_notimpl_warning ("-nomime");
-      mime_encaps = 0;
-      break;
-      
-    case ARG_MSGID:
-      append_msgid = is_true (arg);
-      break;
-      
-    case ARG_NOMSGID:
-      append_msgid = 0;
-      break;
-
-    case ARG_PRESERVE:
-      keep_files = is_true (arg);
-      break;
-      
-    case ARG_PUSH:
-      background = is_true (arg);
-      break;
-      
-    case ARG_NOPUSH:
-      background = 0;
-      break;
-      
-    case ARG_SPLIT:
-      split_message = 1;
-      split_interval = strtoul (arg, &p, 10);
-      if (*p)
-	{
-	  argp_error (state, "%s: %s", arg, _("invalid number"));
-	  exit (1);
-	}
-      break;
-      
-    case ARG_VERBOSE:
-      verbose = is_true (arg);
-      break;
-      
-    case ARG_NOVERBOSE:
-      verbose = 0;
-      break;
-      
-    case ARG_WATCH:
-      watch = is_true (arg);
-      break;
-      
-    case ARG_NOWATCH:
-      watch = 0;
-      break;
-      
-    case ARG_WIDTH:
-      mh_opt_notimpl_warning ("-width");
-      width = strtoul (arg, &p, 10);
-      if (*p)
-	{
-	  argp_error (state, _("invalid number"));
-	  exit (1);
-	}
-      break;
-      
-    default:
-      return ARGP_ERR_UNKNOWN;
-    }
-  return 0;
+  mh_alias_read (arg, 1);
 }
 
+static void
+set_draftfolder (struct mu_parseopt *po, struct mu_option *opt,
+		 char const *arg)
+{
+  draftfolder = mu_strdup (arg);
+  use_draftfolder = 1;
+}
+
+static void
+add_draftmessage (struct mu_parseopt *po, struct mu_option *opt,
+		  char const *arg)
+{
+  add_file (arg);
+}
+
+static void
+set_split_opt (struct mu_parseopt *po, struct mu_option *opt,
+	       char const *arg)
+{
+  char *errmsg;
+  int rc = mu_str_to_c (arg, opt->opt_type, opt->opt_ptr, &errmsg);
+  if (rc)
+    {
+      char const *errtext;
+      if (errmsg)
+	errtext = errmsg;
+      else
+	errtext = mu_strerror (rc);
+
+      mu_parseopt_error (po, "%s%s: %s", po->po_long_opt_start,
+			 opt->opt_long, errtext);
+      free (errmsg);
+
+      if (!(po->po_flags & MU_PARSEOPT_NO_ERREXIT))
+	exit (po->po_exit_error);
+    }
+  split_message = 1;
+}
+
+static struct mu_option options[] = {
+  { "alias",        0,    N_("FILE"), MU_OPTION_DEFAULT,
+    N_("specify additional alias file"),
+    mu_c_string, NULL, add_alias },
+  { "draft",        0,    NULL, MU_OPTION_DEFAULT,
+    N_("use prepared draft"),
+    mu_c_bool, &use_draft },
+  { "draftfolder",  0,   N_("FOLDER"), MU_OPTION_DEFAULT,
+    N_("specify the folder for message drafts"),
+    mu_c_string, NULL, set_draftfolder },
+  { "draftmessage", 0,   N_("MSG"), MU_OPTION_DEFAULT,
+    N_("use MSG from the draftfolder as a draft"),
+    mu_c_string, NULL, add_draftmessage },
+  { "nodraftfolder", 0, NULL, MU_OPTION_DEFAULT,
+    N_("undo the effect of the last --draftfolder option"),
+    mu_c_int, &use_draftfolder, NULL, "1" },
+  { "filter",        0, N_("FILE"), MU_OPTION_HIDDEN,
+    N_("use filter FILE to preprocess the body of the message"),
+    mu_c_string, NULL, mh_opt_notimpl },
+  { "nofilter",      0, NULL, MU_OPTION_HIDDEN,
+    N_("undo the effect of the last --filter option"),
+    mu_c_int, NULL, mh_opt_notimpl },
+  { "format",        0, NULL, MU_OPTION_HIDDEN,
+    N_("reformat To: and Cc: addresses"),
+    mu_c_bool, NULL, mh_opt_notimpl_warning },
+  { "noformat",      0, NULL, MU_OPTION_HIDDEN },
+  { "forward",       0, NULL, MU_OPTION_HIDDEN,
+    N_("in case of failure forward the draft along with the failure notice to the sender"),
+    mu_c_bool, NULL, mh_opt_notimpl_warning },
+  { "noforward",     0, NULL, MU_OPTION_HIDDEN, "" },
+  { "mime",          0, NULL, MU_OPTION_HIDDEN,
+    N_("use MIME encapsulation"),
+    mu_c_bool, NULL, mh_opt_notimpl_warning },
+  { "msgid",         0, NULL, MU_OPTION_DEFAULT,
+    N_("add Message-ID: field"),
+    mu_c_bool, &append_msgid },
+  { "push",          0, NULL, MU_OPTION_DEFAULT,
+    N_("run in the background"),
+    mu_c_bool, &background },
+  { "preserve",      0, NULL, MU_OPTION_DEFAULT,
+    N_("keep draft files"),
+    mu_c_bool, &keep_files },
+  { "keep",          0, NULL, MU_OPTION_ALIAS },
+  { "split",         0, N_("SECONDS"), MU_OPTION_DEFAULT,
+   N_("split the draft into several partial messages and send them with SECONDS interval"),
+    mu_c_ulong, &split_interval, set_split_opt },
+  { "chunksize",     0, N_("NUMBER"), MU_OPTION_DEFAULT,
+    N_("set the size of chunk for --split (in bytes)"),
+    mu_c_size, &split_size },
+  { "verbose",       0, NULL, MU_OPTION_DEFAULT,
+    N_("print the transcript of interactions with the transport system"),
+    mu_c_bool, &verbose },
+  { "watch",         0, NULL, MU_OPTION_DEFAULT,
+    N_("monitor the delivery of mail"),
+    mu_c_bool, &watch },
+  { "width",         0, N_("NUMBER"), MU_OPTION_HIDDEN,
+    N_("make header fields no longer than NUMBER columns"),
+    mu_c_uint, NULL, mh_opt_notimpl_warning },
+
+  MU_OPTION_END
+};
+
 static void
 watch_printf (const char *fmt, ...)
 {
@@ -284,30 +181,58 @@ static mu_list_t mesg_list;
 static mu_property_t mts_profile;
 
 int
-check_file (char *name)
+add_file (char const *name)
 {
   struct list_elt *elt;
-  mu_message_t msg;
-  char *file_name = mh_expand_name (draftfolder, name, 0);
-  
-  msg = mh_file_to_message (NULL, file_name);
-  if (!msg)
-    {
-      free (file_name);
-      return 1;
-    }
+
   if (!mesg_list && mu_list_create (&mesg_list))
     {
-      free (file_name);
       mu_error (_("cannot create message list"));
       return 1;
     }
+  
   elt = mu_alloc (sizeof *elt);
-  elt->file_name = file_name;
-  elt->msg = msg;
+  elt->file_name = name;
+  elt->msg = NULL;
   return mu_list_append (mesg_list, elt);
 }
 
+int
+checkdraft (const char *name)
+{
+  struct stat st;
+  
+  if (stat (name, &st))
+    {
+      mu_error (_("unable to stat draft file %s: %s"), name,
+		mu_strerror (errno));
+      return 1;
+    }
+  return 0;
+}
+
+int
+elt_fixup (void *item, void *data)
+{
+  struct list_elt *elt = item;
+  
+  elt->file_name = mh_expand_name (draftfolder, elt->file_name, NAME_ANY);
+  if (checkdraft (elt->file_name))
+    exit (1);
+  elt->msg = mh_file_to_message (NULL, elt->file_name);
+  if (!elt->msg)
+    return MU_ERR_USER0;
+
+  return 0;
+}
+
+void
+mesg_list_fixup ()
+{
+  if (mesg_list && mu_list_foreach (mesg_list, elt_fixup, NULL))
+    exit (1);
+}  
+
 void
 read_mts_profile ()
 {
@@ -323,7 +248,7 @@ read_mts_profile ()
     {
       mu_property_t local_profile;
 
-      name = mh_expand_name (MHLIBDIR, "mtstailor", 0);
+      name = mh_expand_name (MHLIBDIR, "mtstailor", NAME_ANY);
       mts_profile = mh_read_property_file (name, 1);
 
       name = mu_tilde_expansion ("~/.mtstailor", MU_HIERARCHY_DELIMITER, NULL);
@@ -352,19 +277,9 @@ read_mts_profile ()
   rc = mu_property_sget_value (mts_profile, "localdomain", &p);
   if (rc == 0)
     {
-      char *newdomain;
-
-      newdomain = mu_alloc (strlen (hostname) + 1 + strlen (p) + 1);
-      strcpy (newdomain, hostname);
-      strcat (newdomain, ".");
-      strcat (newdomain, p);
-      rc = mu_set_user_email_domain (newdomain);
-      free (newdomain);
-      if (rc)
-	{
-	  mu_error (_("cannot set user mail domain: %s"), mu_strerror (rc));
-	  exit (1);
-	}
+      hostname = mu_realloc (hostname, strlen (hostname) + 1 + strlen (p) + 1);
+      strcat (hostname, ".");
+      strcat (hostname, p);
     }
   else if (rc != MU_ERR_NOENT)
     {
@@ -372,6 +287,14 @@ read_mts_profile ()
 		       "localdomain", rc);
       exit (1);
     }
+
+  rc = mu_set_user_email_domain (hostname);
+  free (hostname);
+  if (rc)
+    {
+      mu_error (_("cannot set user mail domain: %s"), mu_strerror (rc));
+      exit (1);
+    }
   
   rc = mu_property_sget_value (mts_profile, "username", &p);
   if (rc == 0)
@@ -439,7 +362,7 @@ open_mailer ()
   status = mu_mailer_open (mailer, MU_STREAM_RDWR);
   if (status)
     {
-      mu_error (_("cannot open mailer `%s'"), url);
+      mu_error (_("cannot open mailer `%s': %s"), url, mu_strerror (status));
       return NULL;
     }
   return mailer;
@@ -639,9 +562,13 @@ backup_file (const char *file_name)
 
   if (unlink (new_name) && errno != ENOENT)
     mu_diag_funcall (MU_DIAG_ERROR, "unlink", new_name, errno);
-  else if (rename (file_name, new_name))
-    mu_error (_("cannot rename `%s' to `%s': %s"),
-	      file_name, new_name, mu_strerror (errno));
+  else
+    {
+      int rc = mu_rename_file (file_name, new_name, MU_COPY_OVERWRITE);
+      if (rc)
+	mu_error (_("cannot rename %s to %s: %s"),
+		  file_name, new_name, mu_strerror (errno));
+    }
   free (new_name);
 }
 
@@ -696,7 +623,7 @@ _action_send (void *item, void *data)
 	    mu_header_set_value (hdr, MU_HEADER_X_MAILER,
 				 DEFAULT_X_MAILER, 0);
 	  else if (strcmp (p, "no"))
-	    mu_header_set_value (hdr, MU_HEADER_X_MAILER, p, 0);
+	    mu_header_remove (hdr, MU_HEADER_X_MAILER, 1);
 	}
     }
   
@@ -706,7 +633,7 @@ _action_send (void *item, void *data)
   
   mailer = open_mailer ();
   if (!mailer)
-    return 1;
+    return MU_ERR_FAILURE;
 
   WATCH ((_("Sending message %s"), elt->file_name));
   if (split_message)
@@ -723,7 +650,7 @@ _action_send (void *item, void *data)
   if (rc)
     {
       mu_error(_("cannot send message: %s"), mu_strerror (rc));
-      return 1;
+      return MU_ERR_FAILURE;
     }
 
   WATCH ((_("Destroying the mailer")));
@@ -743,11 +670,25 @@ _add_to_mesg_list (size_t num, mu_message_t msg, void *data)
   struct list_elt *elt;
   size_t uid;
   int rc;
+  char *file_name;
+  
+  if (!mesg_list && mu_list_create (&mesg_list))
+    {
+      mu_error (_("cannot create message list"));
+      return 1;
+    }
+
+  mu_message_get_uid (msg, &uid);
+  file_name = mu_make_file_name (path, mu_umaxtostr (0, uid));
+  if (!use_draft)
+    {
+      if (!mh_usedraft (file_name))
+	exit (0);
+    }
   
   elt = mu_alloc (sizeof *elt);
   elt->msg = msg;
-  mu_message_get_uid (msg, &uid);
-  elt->file_name = mu_make_file_name (path, mu_umaxtostr (0, uid));
+  elt->file_name = file_name;
   rc = mu_list_append (mesg_list, elt);
   if (rc)
     {
@@ -757,74 +698,111 @@ _add_to_mesg_list (size_t num, mu_message_t msg, void *data)
   return 0;
 }
 
+static void
+addfolder (const char *folder, int argc, char **argv)
+{
+  mu_url_t url;
+  const char *path;
+  mu_msgset_t msgset;
+  
+  mu_mailbox_t mbox = mh_open_folder (folder, MU_STREAM_READ);
+  if (!mbox)
+    {
+      mu_error (_("cannot open folder %s: %s"), folder,
+		mu_strerror (errno));
+      exit (1);
+    }
+
+  mh_msgset_parse (&msgset, mbox, argc, argv, "cur");
+  if (!use_draft)
+    {
+      size_t count = 0;
+      mu_msgset_count (msgset, &count);
+      if (count > 1)
+	use_draft = 1;
+    }
+
+  mu_mailbox_get_url (mbox, &url);
+  mu_url_sget_path (url, &path);
+  mu_msgset_foreach_message (msgset, _add_to_mesg_list, (void*)path);
+      
+  mu_msgset_free (msgset);
+}
+
+/* Usage cases:
+ *
+ * 1. send
+ *   a) If Draft-Folder is set: ask whether to use "cur" message from that
+ *      folder as a draft;
+ *   b) If Draft-Folder is not set: ask whether to use $(Path)/draft;
+ * 2. send -draft
+ *   Use $(Path)/draft
+ * 3. send MSG
+ *   Use $(Path)/MSG
+ * 4. send -draftmessage MSG
+ *   Same as (3)
+ * 5. send -draftfolder DIR
+ *   Use "cur" from that folder
+ * 6. send -draftfolder DIR MSG
+ *   Use MSG from folder DIR
+ * 7. send -draftfolder DIR -draftmessage MSG
+ *   Same as 6.
+ */
+
 int
 main (int argc, char **argv)
 {
   mu_mailbox_t mbox = NULL;
-  int index;
   char *p;
   int rc;
   
-  MU_APP_INIT_NLS ();
-  
-  mh_argp_init ();
-  mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
-		 opt_handler, NULL, &index);
+  mh_getopt (&argc, &argv, options, 0, args_doc, prog_doc, NULL);
 
   mh_read_aliases ();
   /* Process the mtstailor file */
   read_mts_profile ();
  
-  argc -= index;
-  argv += index;
-
-  if (draftfolder)
+  if (!draftfolder)
     {
-      mu_msgset_t msgset;
-      mu_url_t url;
-      const char *path;
-      
-      mbox = mh_open_folder (draftfolder, MU_STREAM_RDWR|MU_STREAM_CREAT);
-      mh_msgset_parse (&msgset, mbox, argc, argv, draftmessage);
-      mu_mailbox_get_url (mbox, &url);
-      mu_url_sget_path (url, &path);
-      if ((rc = mu_list_create (&mesg_list)))
+      if (mu_list_is_empty (mesg_list) && argc == 0)
 	{
-	  mu_error (_("cannot create message list: %s"), mu_strerror (rc));
-	  exit (1);
+	  char *dfolder =
+	    (!use_draft && use_draftfolder)
+	                ? (char*) mh_global_profile_get ("Draft-Folder", NULL)
+	                : NULL;
+
+	  if (dfolder)
+	    addfolder (dfolder, 0, NULL);
+	  else
+	    {
+	      char *df = mh_expand_name (mu_folder_directory (), "draft", 
+	                                 NAME_ANY);
+	      if (checkdraft (df))
+		exit (1);
+	      if (!use_draft && !mh_usedraft (df))
+		exit (0);
+	      add_file (df);
+	      mesg_list_fixup ();
+	    }
+	}
+      else
+	{
+	  while (argc--)
+	    add_file (*argv++);
+	  mesg_list_fixup ();
 	}
-      mu_msgset_foreach_message (msgset, _add_to_mesg_list, (void*)path);
-      
-      mu_msgset_free (msgset);
     }
   else
     {
-      int i;
-      
-      if (argc == 0)
-	{
-	  char *xargv[2];
-	  struct stat st;
-	  
-	  xargv[0] = mh_draft_name ();
-
-	  if (stat (xargv[0], &st))
-	    {
-	      mu_diag_funcall (MU_DIAG_ERROR, "stat", xargv[0], errno);
-	      return 1;
-	    }
-
-	  if (!use_draft && !mh_usedraft (xargv[0]))
-	    exit (0);
-	  xargv[1] = NULL;
-	  argv = xargv;
-	  argc = 1;
-	}
-      for (i = 0; i < argc; i++)
-	if (check_file (argv[i]))
-	  return 1;
+      /* -draftfolder is supplied */
+      draftfolder = mh_expand_name (mu_folder_directory (),
+				    draftfolder, NAME_ANY);
+      use_draft = 1;
+      mesg_list_fixup ();
+      if (mu_list_is_empty (mesg_list) || argc != 0)
+	addfolder (draftfolder, argc, argv);
     }
-
+  
   /* Detach from the console if required */
   if (background && daemon (0, 0) < 0)
     {
