@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007, 2010-2012, 2014-2017 Free Software
+   Copyright (C) 2005, 2007, 2010-2012, 2014-2018 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -22,6 +22,8 @@
 # include <strings.h>
 #endif
 #include <mailutils/mailutils.h>
+#include <mailutils/locus.h>
+#include <mailutils/yyloc.h>
 #include <fnmatch.h>
 
 struct mimetypes_string
@@ -36,21 +38,15 @@ int mimetypes_yyerror (char *s);
 int mimetypes_open (const char *name);
 void mimetypes_close (void);
 int mimetypes_parse (const char *name);
-void mimetypes_gram_debug (int level);
-void mimetypes_lex_debug (int level);
 void mimetypes_lex_init (void);
-void reset_lex (void);
+
+void lex_next_rule (void);
 void *mimetypes_malloc (size_t size);
 
-struct mimetypes_string mimetypes_append_string2 (struct mimetypes_string *s1,
-						  char c,
-						  struct mimetypes_string *s2);
 struct mimetypes_string *mimetypes_string_dup (struct mimetypes_string *s);
 
 const char *get_file_type (void);
 
-extern char *mimeview_file;
+extern char const *mimeview_file;
 extern mu_stream_t mimeview_stream;    
-extern int debug_level;
 
-#define DEBUG(l,f) if (debug_level > (l)) printf f
