@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2005, 
-   2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2003, 2005, 2007-2012, 2014-2018 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,12 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "pop3d.h"
-#include "mailutils/libargp.h"
+#include "mailutils/property.h"
 
-static mu_stream_t istream, ostream;
+mu_stream_t iostream;
 
 void
 pop3d_parse_command (char *cmd, char **pcmd, char **parg)
@@ -51,9 +49,9 @@ pop3d_abquit (int reason)
   /* Unlock spool */
   if (state != AUTHORIZATION)
     {
-      pop3d_unlock ();
       mu_mailbox_flush (mbox, 0);
       mu_mailbox_close (mbox);
+      manlock_unlock (mbox);
       mu_mailbox_destroy (&mbox);
     }
 
@@ -61,7 +59,7 @@ pop3d_abquit (int reason)
     {
     case ERR_NO_MEM:
       code = EX_SOFTWARE;
-      pop3d_outf ("-ERR %s\r\n", pop3d_error_string (reason));
+      pop3d_outf ("-ERR %s\n", pop3d_error_string (reason));
       mu_diag_output (MU_DIAG_ERROR, _("not enough memory"));
       break;
 
@@ -77,7 +75,7 @@ pop3d_abquit (int reason)
 
     case ERR_TIMEOUT:
       code = EX_TEMPFAIL;
-      pop3d_outf ("-ERR %s\r\n", pop3d_error_string (reason));
+      pop3d_outf ("-ERR %s\n", pop3d_error_string (reason));
       if (state == TRANSACTION)
 	mu_diag_output (MU_DIAG_INFO, _("session timed out for user: %s"),
 			username);
@@ -95,6 +93,10 @@ pop3d_abquit (int reason)
       mu_diag_output (MU_DIAG_INFO, _("no socket to send to"));
       break;
 
+    case ERR_FILE:
+      code = EX_IOERR;
+      break;
+      
     case ERR_PROTO:
       code = EX_PROTOCOL;
       mu_diag_output (MU_DIAG_INFO, _("remote protocol error"));
@@ -111,12 +113,12 @@ pop3d_abquit (int reason)
 		      _("mailbox was updated by other party: %s"),
 		      username);
       pop3d_outf
-	("-ERR [OUT-SYNC] Mailbox updated by other party or corrupt\r\n");
+	("-ERR [OUT-SYNC] Mailbox updated by other party or corrupt\n");
       break;
 
     default:
       code = EX_SOFTWARE;
-      pop3d_outf ("-ERR Quitting: %s\r\n", pop3d_error_string (reason));
+      pop3d_outf ("-ERR Quitting: %s\n", pop3d_error_string (reason));
       mu_diag_output (MU_DIAG_ERROR, _("quitting (numeric reason %d)"),
 		      reason);
       break;
@@ -126,114 +128,175 @@ pop3d_abquit (int reason)
   exit (code);
 }
 
-void
-pop3d_setio (FILE *in, FILE *out)
+static void
+log_cipher (mu_stream_t stream)
 {
-  if (!in)
-    pop3d_abquit (ERR_NO_IFILE);
-  if (!out)
-    pop3d_abquit (ERR_NO_OFILE);
-
-  setvbuf (in, NULL, _IOLBF, 0);
-  setvbuf (out, NULL, _IOLBF, 0);
-  if (mu_stdio_stream_create (&istream, in, MU_STREAM_NO_CLOSE))
-    pop3d_abquit (ERR_NO_IFILE);
-  if (mu_stdio_stream_create (&ostream, out, MU_STREAM_NO_CLOSE))
-    pop3d_abquit (ERR_NO_OFILE);
+  mu_property_t prop;
+  int rc = mu_stream_ioctl (stream, MU_IOCTL_TLSSTREAM,
+			    MU_IOCTL_TLS_GET_CIPHER_INFO, &prop);
+  if (rc)
+    {
+      mu_diag_output (MU_DIAG_INFO, _("TLS established"));
+      mu_diag_output (MU_DIAG_ERROR, _("can't get TLS details: %s"),
+		      mu_strerror (rc));
+    }
+  else
+    {
+      char const *cipher, *mac, *proto;
+      if (mu_property_sget_value (prop, "cipher", &cipher))
+	cipher = "UNKNOWN";	
+      if (mu_property_sget_value (prop, "mac", &mac))
+	mac = "UNKNOWN";
+      if (mu_property_sget_value (prop, "protocol", &proto))
+	proto = "UNKNOWN";
+      
+      mu_diag_output (MU_DIAG_INFO, _("TLS established using %s-%s (%s)"),
+		      cipher, mac, proto);
+      
+      mu_property_destroy (&prop);
+    }
 }
 
-#ifdef WITH_TLS
-int
-pop3d_init_tls_server ()
+void
+pop3d_setio (int ifd, int ofd, struct mu_tls_config *tls_conf)
 {
-  mu_stream_t stream;
-  int rc;
- 
-  rc = mu_tls_stream_create (&stream, istream, ostream, 0);
-  if (rc)
-    return 0;
+  mu_stream_t str, istream, ostream;
+  
+  if (ifd == -1)
+    pop3d_abquit (ERR_NO_IFILE);
+  if (ofd == -1)
+    pop3d_abquit (ERR_NO_OFILE);
 
-  if (mu_stream_open (stream))
+  if (mu_stdio_stream_create (&istream, ifd, MU_STREAM_READ))
+    pop3d_abquit (ERR_NO_IFILE);
+  mu_stream_set_buffer (istream, mu_buffer_line, 0);
+
+  if (mu_stdio_stream_create (&ostream, ofd, MU_STREAM_WRITE))
+    pop3d_abquit (ERR_NO_OFILE);
+
+  /* Combine the two streams into an I/O one. */
+  if (tls_conf)
     {
-      const char *p;
-      mu_stream_strerror (stream, &p);
-      mu_diag_output (MU_DIAG_ERROR, _("cannot open TLS stream: %s"), p);
-      return 0;
+      int rc = mu_tls_stream_create (&str, istream, ostream,
+				     tls_conf,
+				     MU_TLS_SERVER,
+				     0);
+      if (rc)
+	{
+	  mu_stream_unref (istream);
+	  mu_stream_unref (ostream);
+	  mu_error (_("failed to create TLS stream: %s"), mu_strerror (rc));
+	  pop3d_abquit (ERR_FILE);
+	}
+      log_cipher (str);
+    }
+  else if (mu_iostream_create (&str, istream, ostream))
+    pop3d_abquit (ERR_FILE);
+
+  /* Convert all writes to CRLF form.
+     There is no need to convert reads, as the code ignores extra \r anyway.
+  */
+  if (mu_filter_create (&iostream, str, "CRLF", MU_FILTER_ENCODE,
+			MU_STREAM_WRITE | MU_STREAM_RDTHRU))
+    pop3d_abquit (ERR_NO_IFILE);
+  /* Change buffering scheme: filter streams are fully buffered by default. */
+  mu_stream_set_buffer (iostream, mu_buffer_line, 0);
+  
+  if (pop3d_transcript)
+    {
+      int rc;
+      mu_stream_t dstr, xstr;
+      
+      rc = mu_dbgstream_create (&dstr, MU_DIAG_DEBUG);
+      if (rc)
+	mu_error (_("cannot create debug stream; transcript disabled: %s"),
+		  mu_strerror (rc));
+      else
+	{
+	  rc = mu_xscript_stream_create (&xstr, iostream, dstr, NULL);
+	  mu_stream_unref (dstr);
+	  if (rc)
+	    mu_error (_("cannot create transcript stream: %s"),
+		      mu_strerror (rc));
+	  else
+	    {
+	      mu_stream_unref (iostream);
+	      iostream = xstr;
+	    }
+	}
+    }
+}
+
+int
+pop3d_init_tls_server (struct mu_tls_config *tls_conf)
+{
+  mu_stream_t tlsstream, stream[2];
+  int rc;
+
+  rc = mu_stream_ioctl (iostream, MU_IOCTL_SUBSTREAM, MU_IOCTL_OP_GET, stream);
+  if (rc)
+    {
+      mu_error (_("%s failed: %s"), "MU_IOCTL_GET_STREAM",
+		mu_stream_strerror (iostream, rc));
+      return 1;
     }
   
-  istream = ostream = stream;
-  return 1;
+  rc = mu_tls_stream_create (&tlsstream, stream[0], stream[1],
+			     tls_conf,
+			     MU_TLS_SERVER,
+			     0);
+  mu_stream_unref (stream[0]);
+  mu_stream_unref (stream[1]);
+  if (rc)
+    return 1;
+
+  log_cipher (tlsstream);
+
+  stream[0] = stream[1] = tlsstream;
+  rc = mu_stream_ioctl (iostream, MU_IOCTL_SUBSTREAM, MU_IOCTL_OP_SET, stream);
+  mu_stream_unref (stream[0]);
+  mu_stream_unref (stream[1]);
+  if (rc)
+    {
+      mu_error (_("%s failed: %s"), "MU_IOCTL_SET_STREAM",
+		mu_stream_strerror (iostream, rc));
+      pop3d_abquit (ERR_IO);
+    }
+  return 0;
 }
-#endif
 
 void
 pop3d_bye ()
 {
-  if (istream == ostream)
-    {
-      mu_stream_close (istream);
-      mu_stream_destroy (&istream, mu_stream_get_owner (istream));
-    }
-  /* There's no reason closing in/out streams otherwise */
-#ifdef WITH_TLS
-  if (tls_available)
-    mu_deinit_tls_libs ();
-#endif /* WITH_TLS */
+  mu_stream_close (iostream);
+  mu_stream_destroy (&iostream);
 }
 
 void
 pop3d_flush_output ()
 {
-  mu_stream_flush (ostream);
+  mu_stream_flush (iostream);
 }
 
 int
 pop3d_is_master ()
 {
-  return ostream == NULL;
-}
-
-static void
-transcript (const char *pfx, const char *buf)
-{
-  if (pop3d_transcript)
-    {
-      int len = strlen (buf);
-      if (len > 0 && buf[len-1] == '\n')
-	{
-	  len--;
-	  if (len > 0 && buf[len-1] == '\r')
-	    len--;
-	}
-      mu_diag_output (MU_DIAG_DEBUG, "%s: %-.*s", pfx, len, buf);
-    }
+  return iostream == NULL;
 }
 
 void
 pop3d_outf (const char *fmt, ...)
 {
   va_list ap;
-  char *buf;
   int rc;
   
   va_start (ap, fmt);
-  vasprintf (&buf, fmt, ap);
+  rc = mu_stream_vprintf (iostream, fmt, ap);
   va_end (ap);
-
-  if (!buf)
-    pop3d_abquit (ERR_NO_MEM);
-  
-  transcript ("sent", buf);
-
-  rc = mu_stream_sequential_write (ostream, buf, strlen (buf));
-  free (buf);
   if (rc)
     {
-      const char *p;
-
-      if (mu_stream_strerror (ostream, &p))
-	p = strerror (errno);
-      mu_diag_output (MU_DIAG_ERROR, _("write failed: %s"), p);
+      mu_diag_output (MU_DIAG_ERROR, _("Write failed: %s"),
+ 		      mu_stream_strerror (iostream, rc));
       pop3d_abquit (ERR_IO);
     }
 }
@@ -246,16 +309,13 @@ pop3d_readline (char *buffer, size_t size)
   size_t nbytes;
   
   alarm (idle_timeout);
-  rc = mu_stream_sequential_readline (istream, buffer, size, &nbytes);
+  rc = mu_stream_readline (iostream, buffer, size, &nbytes);
   alarm (0);
 
   if (rc)
     {
-      const char *p;
-
-      if (mu_stream_strerror (ostream, &p))
-	p = strerror (errno);
-      mu_diag_output (MU_DIAG_ERROR, _("read failed: %s"), p);
+      mu_diag_output (MU_DIAG_ERROR, _("Read failed: %s"),
+ 		      mu_stream_strerror (iostream, rc));
       pop3d_abquit (ERR_IO);
     }
   else if (nbytes == 0)
@@ -269,8 +329,6 @@ pop3d_readline (char *buffer, size_t size)
       pop3d_abquit (ERR_PROTO);
     }
 
-  transcript ("recv", buffer);
-
   return buffer;
 }
 
@@ -314,3 +372,23 @@ pop3d_undelete_all ()
        mu_attribute_unset_deleted (attr);
     }
 }
+
+int
+set_xscript_level (int xlev)
+{
+  if (pop3d_transcript)
+    {
+      if (xlev != MU_XSCRIPT_NORMAL)
+	{
+	  if (mu_debug_level_p (MU_DEBCAT_REMOTE,
+	                        xlev == MU_XSCRIPT_SECURE ?
+	                            MU_DEBUG_TRACE6 : MU_DEBUG_TRACE7))
+	    return MU_XSCRIPT_NORMAL;
+	}
+
+      if (mu_stream_ioctl (iostream, MU_IOCTL_XSCRIPTSTREAM,
+                           MU_IOCTL_XSCRIPTSTREAM_LEVEL, &xlev) == 0)
+	return xlev;
+    }
+  return 0;
+}
