@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005, 2007, 2009-2012 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2001, 2005, 2007, 2009-2012, 2014-2018 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@
   XLINES capability.
 */
 int
-pop3d_list (char *arg)
+pop3d_list (char *arg, struct pop3d_session *sess)
 {
   size_t mesgno;
   mu_message_t msg = NULL;
