@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999-2001, 2005, 2007, 2010-2012, 2014-2018 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,7 +20,7 @@
 /* Resets the connection so that no messages are marked as deleted */
 
 int
-pop3d_rset (char *arg)
+pop3d_rset (char *arg, struct pop3d_session *sess)
 {
   size_t i;
   size_t total = 0;
@@ -40,6 +41,6 @@ pop3d_rset (char *arg)
       mu_message_get_attribute (msg, &attr);
       pop3d_unset_deleted (attr);
     }
-  pop3d_outf ("+OK\r\n");
+  pop3d_outf ("+OK\n");
   return OK;
 }
