@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2004-2005, 2007, 2009-2012, 2014-2018 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -23,16 +23,18 @@
 #ifdef HAVE_PGSQL
 #include <mailutils/mailutils.h>
 #include <mailutils/sql.h>
+#include <mailutils/cctype.h>
 
 #include <libpq-fe.h>
-#include <ctype.h>
 
 static char *
 chop (char *str)
 {
   int len;
   
-  for (len = strlen(str); len > 0 && isspace(str[len-1]); len--)
+  if (!str)
+    return NULL;
+  for (len = strlen (str); len > 0 && mu_isspace (str[len-1]); len--)
     ;
   str[len] = 0;
   return str;
