@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2005, 2006, 2007, 2009, 2010 Free Software
+   Copyright (C) 2004-2007, 2009-2012, 2014-2018 Free Software
    Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -13,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see 
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -70,7 +69,7 @@ add_disptab (mu_sql_dispatch_t *tab)
 #endif
 
 int
-mu_sql_interface_index (char *name)
+mu_sql_interface_index (char const *name)
 {
   int i;
   //mu_sql_dispatch_t *tab;
